<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Block\Adminhtml\Grid;

use Magento\Backend\Block\Template\Context;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Model\Config\Source\ProductAttributes;
use WriteTextAI\WriteTextAI\Model\GenerateOptionsManager;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;
use WriteTextAI\WriteTextAI\ViewModel\Tooltip;
use WriteTextAI\WriteTextAI\ViewModel\Permissions;
use WriteTextAI\WriteTextAI\ViewModel\Settings;
use WriteTextAI\WriteTextAI\ViewModel\UserFieldsSelected;

class Prompt extends \Magento\Backend\Block\Template
{
    /**
     * @var string
     */
    protected $_template = 'WriteTextAI_WriteTextAI::grid/prompt.phtml';

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ProductAttributes
     */
    protected $attributes;

    /**
     * @var GenerateOptionsManager
     */
    protected $generateOptionsManager;

    /**
     * @var UserFieldsSelected
     */
    protected $userFieldsSelected;

    /**
     * @var Tooltip
     */
    protected $tooltip;

    /**
     * @var Permissions
     */
    protected $permissions;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Data $helper
     * @param ProductAttributes $attributes
     * @param GenerateOptionsManager $generateOptionsManager
     * @param UserFieldsSelected $userFieldsSelected
     * @param Tooltip $tooltip
     * @param Permissions $permissions
     * @param Settings $settings
     * @param array $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        ProductAttributes $attributes,
        GenerateOptionsManager $generateOptionsManager,
        UserFieldsSelected $userFieldsSelected,
        Tooltip $tooltip,
        Permissions $permissions,
        Settings $settings,
        array $data = []
    ) {
        $this->helper = $helper;
        $this->attributes = $attributes;
        $this->generateOptionsManager = $generateOptionsManager;
        $this->userFieldsSelected = $userFieldsSelected;
        $this->tooltip = $tooltip;
        $this->permissions = $permissions;
        $this->settings = $settings;
        parent::__construct($context, $data);
    }

    /**
     * Is Attribute Allowed
     *
     * @return bool
     */
    public function isAttrAllowed()
    {
        return $this->permissions->isAttrAllowed();
    }

    /**
     * Is Tone Style Allowed
     *
     * @return bool
     */
    public function isToneStyleAllowed()
    {
        return $this->permissions->isToneStyleAllowed();
    }

    /**
     * Is Words Allowed
     *
     * @return bool
     */
    public function isWordsAllowed()
    {
        return $this->permissions->isWordsAllowed();
    }

    /**
     * Get tooltip
     *
     * @param string $block
     * @return string
     */
    public function getTooltip($block)
    {
        return $this->tooltip->getTooltip($block);
    }

    /**
     * Get tones
     *
     * @return array
     */
    public function getTones()
    {
        return $this->generateOptionsManager->getOptions('tones');
    }

    /**
     * Get tones selected
     *
     * @return array
     */
    public function getTonesSelected()
    {
        $tones = $this->getTones();
        $tonesSelected = array_filter($tones, function ($tone) {
            return $tone['default'] === true;
        });
        $tonesSelected = array_column($tonesSelected, 'id');

        return $tonesSelected;
    }

    /**
     * Get styles
     *
     * @return array
     */
    public function getStyles()
    {
        return $this->generateOptionsManager->getOptions('styles');
    }

    /**
     * Get styles selected
     *
     * @return array
     */
    public function getStylesSelected()
    {
        $styles = $this->getStyles();
        $stylesSelected = array_filter($styles, function ($style) {
            return $style['default'] === true;
        });
        $stylesSelected = array_column($stylesSelected, 'id');

        return array_shift($stylesSelected);
    }

    /**
     * Get audience
     *
     * @return array
     */
    public function getAudience()
    {
        return $this->generateOptionsManager->getOptions('audience');
    }

    /**
     * Get audience selected
     *
     * @return array
     */
    public function getAudienceSelected()
    {
        $audience = $this->getAudience();
        $audienceSelected = array_filter($audience, function ($audience) {
            return $audience['default'] === true;
        });
        $audienceSelected = array_column($audienceSelected, 'id');

        return $audienceSelected;
    }

    /**
     * Get attributes
     *
     * @return array
     */
    public function getAttributes()
    {
        return $this->generateOptionsManager->getOptions('attributes');
    }

    /**
     * Get description min
     *
     * @return string
     */
    public function getDescriptionMin()
    {
        return $this->helper->getDescriptionSettings('min');
    }

    /**
     * Get description max
     *
     * @return string
     */
    public function getDescriptionMax()
    {
        return $this->helper->getDescriptionSettings('max');
    }

    /**
     * Get excerpt min
     *
     * @return string
     */
    public function getExcerptMin()
    {
        return $this->helper->getExcerptSettings('min');
    }

    /**
     * Get excerpt max
     *
     * @return string
     */
    public function getExcerptMax()
    {
        return $this->helper->getExcerptSettings('max');
    }

    /**
     * Get rules
     *
     * @return array
     */
    public function getRules()
    {
        return $this->settings->getSettings('rules');
    }

    /**
     * Get disallowed combinations
     *
     * @return array
     */
    public function getDisallowedCombinations()
    {
        return $this->settings->getSettings('disallowedCombinations');
    }
    
    /**
     * Get text types
     *
     * @return array
     */
    public function getFields()
    {
        $fields = [[
            'value' => 'page_title',
            'label' =>  __('Meta title')
        ], [
            'value' => 'page_description',
            'label' =>  __('Meta description')
        ], [
            'value' => 'product_description',
            'label' =>  __('Description')
        ], [
            'value' => 'short_product_description',
            'label' => $this->helper->getCustomOpenGraph() ?
                __('Short description') :  __('Short description / Open Graph text')
        ]];

        if ($this->helper->getCustomOpenGraph()) {
            $fields[] = [
                'value' => 'open_graph',
                'label' => __('Open Graph text')
            ];
        }

        return $fields;
    }

    /**
     * Get fields selected
     *
     * @return array
     */
    public function getFieldsSelected()
    {
        return $this->userFieldsSelected->getUserFieldsSelected(
            UserFieldsSelectedManager::ACTION_TYPE_BULK_GENERATE_FIELDS
        );
    }

    /**
     * Get reference product collection
     *
     * @return array
     */
    public function getReferenceProductCollection()
    {
        return $this->helper->getReferenceProductCollection();
    }
}
