<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\SettingManager;

class Ideas extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     * @param AiProductManager $aiProductManager
     * @param Premium $premium
     * @param SettingManager $settingManager
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager,
        AiProductManager $aiProductManager,
        Premium $premium,
        SettingManager $settingManager
    ) {
        $this->apiManager = $apiManager;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
        $this->aiProductManager = $aiProductManager;
        $this->premium = $premium;
        $this->settingManager = $settingManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $keywords = $this->getRequest()->getParam('keywords');
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $location = $this->getRequest()->getParam('location');
            $filterByCompetition = $this->getRequest()->getParam('competition', []);
            $filterBySearchVolumeMinimum = $this->getRequest()->getParam('search_volume_minimum');
            $filterBySearchVolumeMaximum = $this->getRequest()->getParam('search_volume_maximum');
            $competitionSort = $this->getRequest()->getParam('competition_sort');
            $volumeSort = $this->getRequest()->getParam('volume_sort');
            $keywordSort = $this->getRequest()->getParam('keyword_sort');
            $page = $this->getRequest()->getParam('page');
            $pageSize = $this->getRequest()->getParam('page_size');
            $noGenerate = filter_var($this->getRequest()->getParam('nogenerate'), FILTER_VALIDATE_BOOLEAN);
            $refresh = filter_var($this->getRequest()->getParam('refresh'), FILTER_VALIDATE_BOOLEAN);

            if ($keywords) {
                if ($refresh) {
                    $this->aiProductManager->setRefreshDate($productId, $storeId);
                }

                $rules = $this->settingManager->getSettings('rules');
                $concattedKeywords = array_map(function ($text) use ($rules) {
                    return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
                }, array_column($keywords, 'text'));

                $ideas = $this->apiManager->getKeywordIdeas(
                    $storeId,
                    $productId,
                    $concattedKeywords,
                    $location,
                    $filterByCompetition,
                    $filterBySearchVolumeMinimum,
                    $filterBySearchVolumeMaximum,
                    $competitionSort,
                    $volumeSort,
                    $keywordSort,
                    $page,
                    $pageSize,
                    $refresh,
                    $noGenerate
                );
                
                $ideasKeywords = [];

                if (isset($ideas['keywords'])) {
                    foreach ($ideas['keywords'] as $text) {
                        $ideasKeywords[] = [
                            'text' => $text['keyword'],
                            'searchVolume' => $text['search_volume'] ?? __('No data'),
                            'difficulty' => $text['competition'] ?? __('No data')
                        ];
                    }
                }
                
                $data = array_map(function ($keyword) {
                    unset($keyword['searchVolume']);
                    unset($keyword['difficulty']);
                    return $keyword;
                }, $keywords);
                $this->mergeArraysByProperty($data, $ideasKeywords, 'text');
    
                $this->keywordIdeaFiltersManager->saveProductKeywordIdeasFilters(
                    $storeId,
                    $productId,
                    $location,
                    $filterByCompetition,
                    $filterBySearchVolumeMinimum,
                    $filterBySearchVolumeMaximum,
                    $competitionSort,
                    $volumeSort,
                    $keywordSort
                );
                    
                $result = [
                    'success' => true,
                    'ideas' => $ideas['result'] ?? [],
                    'keywords' => $data,
                    'total_page' => isset($ideas['result_count']) ? ceil($ideas['result_count'] / $pageSize) : 0
                ];
            } else {
                $result = [
                    'success' => true,
                    'ideas' => [],
                    'total_page' => 0,
                    'keywords' => []
                ];
            }
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage(),
                'ideas' => $ideas['result'] ?? [],
                'keywords' => $data ?? [],
                'total_page' => isset($ideas['result_count']) ? ceil($ideas['result_count'] / $pageSize) : 0
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }

    /**
     * Merge arrays by property
     *
     * @param array $array1
     * @param array $array2
     * @param string $matchingProperty
     * @return void
     */
    private function mergeArraysByProperty(&$array1, $array2, $matchingProperty)
    {
        $tempArray = [];
    
        foreach ($array2 as $item) {
            $tempArray[$item[$matchingProperty]] = $item;
        }
    
        foreach ($array1 as &$item) {
            $id = $item[$matchingProperty];
            
            if (isset($tempArray[$id])) {
                $item += $tempArray[$id];
            }
        }
    }
}
