<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use WriteTextAI\WriteTextAI\Model\Api\CurlBuilder;
use Magento\Framework\View\Result\PageFactory;
use WriteTextAI\WriteTextAI\Helper\Data;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultInterface;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Stores as StoresFilterOptions;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;

class Index extends \Magento\Backend\App\Action
{
    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var CurlBuilder
     */
    protected $curlBuilder;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var EventManager
     */
    protected $eventManager;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * Constructor
     *
     * @param Context $context
     * @param PageFactory $resultPageFactory
     * @param CurlBuilder $curlBuilder
     * @param Data $helper
     * @param EventManager $eventManager
     * @param ScopeConfigInterface $scopeConfig
     * @param ApiSession $apiSession
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        CurlBuilder $curlBuilder,
        Data $helper,
        EventManager $eventManager,
        ScopeConfigInterface $scopeConfig,
        ApiSession $apiSession
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->curlBuilder = $curlBuilder;
        $this->helper = $helper;
        $this->eventManager = $eventManager;
        $this->scopeConfig = $scopeConfig;
        $this->apiSession = $apiSession;
        parent::__construct($context);
    }

    /**
     * Index action
     *
     * @return ResultInterface
     */
    public function execute()
    {
        $resultRedirect = $this->resultRedirectFactory->create();
        $locale = $this->apiSession->getCurrentUser()->getInterfaceLocale();

        try {
            if (!$this->isSetupComplete()) {
                return $resultRedirect->setPath('writetextai/setup/index');
            } else {
                $this->eventManager->dispatch('writetextai_prepare_settings');
                $this->isLanguageSupported();
                if (strpos($locale, 'en_') !== 0) {
                    $this->messageManager->addNotice(
                        __("Translation of extension help text is ongoing. Please stay tuned.")
                    );
                }

                $resultPage = $this->resultPageFactory->create();
                $resultPage->getConfig()->getTitle()->prepend(__("WriteText.ai"));
                $resultPage->setActiveMenu('WriteTextAI_WriteTextAI::menu');
                return $resultPage;
            }
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            return $resultRedirect->setPath('writetextai/setup/index');
        } catch (\WriteTextAI\WriteTextAI\Exception\UnsupportedLanguageException $e) {
            $this->messageManager->addError($e->getMessage());
            if (strpos($locale, 'en_') !== 0) {
                $this->messageManager->addNotice(
                    __("Translation of extension help text is ongoing. Please stay tuned.")
                );
            }

            $resultPage = $this->resultPageFactory->create();
            $resultPage->getConfig()->getTitle()->prepend(__("WriteText.ai"));
            $resultPage->setActiveMenu('WriteTextAI_WriteTextAI::menu');
            $resultPage->getLayout()->unsetElement('content');
            $resultPage->getLayout()->unsetElement('header');

            return $resultPage;
        } catch (\WriteTextAI\WriteTextAI\Exception\VersionException $e) {
            if (strpos($locale, 'en_') !== 0) {
                $this->messageManager->addNotice(
                    __("Translation of extension help text is ongoing. Please stay tuned.")
                );
            }
            
            $resultPage = $this->resultPageFactory->create();
            $resultPage->getConfig()->getTitle()->prepend(__("WriteText.ai"));
            $resultPage->setActiveMenu('WriteTextAI_WriteTextAI::menu');
            $resultPage->getLayout()->unsetElement('content');
            $resultPage->getLayout()->unsetElement('header');

            return $resultPage;
        }
    }

    /**
     * Check if language is supported
     *
     * @param string $language
     * @return bool
     */
    protected function isLanguageSupported()
    {
        $localeCode =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store::DEFAULT_STORE_ID
        );

        $languageCode = explode('_', $localeCode)[0];

        if (!in_array($languageCode, StoresFilterOptions::SUPPORTED_LANGUAGES)) {
            // phpcs:ignore Generic.Files.LineLength.TooLong
            throw new \WriteTextAI\WriteTextAI\Exception\UnsupportedLanguageException(__('WriteText.ai beta is currently available for certain languages only. Please install this extension in a website with a supported language (EN, DA, DE, NO, SV, FR, ES, PT, NL, CA, IT) to proceed with beta testing.'));
        }
    }

    /**
     * Check if setup is complete
     *
     * @return bool
     */
    private function isSetupComplete()
    {
        $accountToken = $this->helper->getAccountToken();
        $region = $this->helper->getRegion();
        
        /**
         * check if account token and region is set
         * then verify account token
         */
        if ($accountToken &&
            $region &&
            $this->curlBuilder->getConnectToken($accountToken)
        ) {
            // Check if all text fields are mapped to an existing attribute
            if ($this->helper->isDataMapped()) {
                // Check if final step is done and finalized
                if ($this->helper->isFinalStepDone() && $this->helper->isSetupFinalized()) {
                    return true;
                }
            } else {
                if ($this->helper->isSetupFinalized()) {
                    $notMappedField = $this->helper->getNotMappedField();
                    // phpcs:ignore Generic.Files.LineLength.TooLong
                    $this->messageManager->addError(sprintf(__("You have been redirected here because we detected a change in the field that you have set for %s. You will need to map a text field so WriteText.ai has a place to transfer your %s to. Don't worry, all your content is saved and you will be redirected back to your dashboard after setting this up."), $notMappedField, $notMappedField));
                }
            }
        }

        return false;
    }
}
