<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\UrlInterface;
use WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Helper\Image as WtaImageHelper;
use Magento\Catalog\Helper\Image as ImageHelper;
use Magento\Framework\Exception\FileSystemException;

class UploadThumbnails extends Action
{
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var WtaImageHelper
     */
    protected $wtaImageHelper;

    /**
     * @var ImageHelper
     */
    protected $imageHelper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param UrlInterface $urlBuilder
     * @param ProductRepositoryInterface $productRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param RemoteAddress $remoteAddress
     * @param WtaImageHelper $wtaImageHelper
     * @param ImageHelper $imageHelper
     */
    public function __construct(
        Context $context,
        UrlInterface $urlBuilder,
        ProductRepositoryInterface $productRepository,
        ScopeConfigInterface $scopeConfig,
        RemoteAddress $remoteAddress,
        WtaImageHelper $wtaImageHelper,
        ImageHelper $imageHelper
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->productRepository = $productRepository;
        $this->scopeConfig = $scopeConfig;
        $this->remoteAddress = $remoteAddress;
        $this->wtaImageHelper = $wtaImageHelper;
        $this->imageHelper = $imageHelper;

        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $uploadResult = $this->uploadThumbnails();
            $result = [
                'success' => true,
                'failed_thumbnails' => $uploadResult['failed_thumbnails'] ?? [],
                'failed_products' => $uploadResult['failed_products'] ?? [],
                'success_thumbnails' => $uploadResult['success_thumbnails'] ?? []
            ];
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('writetextai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Upload thumbnails
     */
    protected function uploadThumbnails()
    {
        $selected = $this->getRequest()->getParam('selected', []);
        $storeId = $this->getRequest()->getParam('store_id', Store::DEFAULT_STORE_ID);
        $locale =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
        $language = str_replace('_', '-', $locale);
        $ipAddress = $this->remoteAddress->getRemoteAddress();

        $failedThumbnails = [];
        $failedProducts = [];
        $successThumbnails = [];

        foreach ($selected as $productId) {
            $product = $this->productRepository->getById($productId, false, $storeId);

            $thumbnailId = $this->wtaImageHelper->getImageValue($product);
            if (empty($thumbnailId)) {
                continue;
            }

            $imageHelper = $this->imageHelper->init($product, 'product_listing_thumbnail');
            try {
                $thumbnail = $imageHelper->setImageFile($product->getData('thumbnail'));
                $thumbnailUrl = $thumbnail->getUrl();
                $thumbnail->getOriginalWidth();
            } catch (\InvalidArgumentException | FileSystemException $e) {
                continue;
            }

            try {
                $successThumbnails[] = $this->wtaImageHelper->getImageApiId(
                    $storeId,
                    $ipAddress,
                    $language,
                    $thumbnailId,
                    $thumbnailUrl
                );
            } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
                $failedProducts[] = $productId;
                $failedThumbnails[] = $thumbnailUrl;
            } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
                $failedProducts[] = $productId;
                $failedThumbnails[] = $thumbnailUrl;
            }
        }

        return [
            'failed_thumbnails' => $failedThumbnails,
            'failed_products' => $failedProducts,
            'success_thumbnails' => $successThumbnails
        ];
    }
}
