<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Image;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Helper\Image;
use Magento\Framework\UrlInterface;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class Upload extends Action
{
    /**
     * @var Image
     */
    protected $imageHelper;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var Settings
     */
    protected $settings;
    /**
     * Constructor
     *
     * @param Context $context
     * @param Image $imageHelper
     * @param UrlInterface $urlBuilder
     * @param Premium $premium
     * @param ScopeConfigInterface $scopeConfig
     * @param RemoteAddress $remoteAddress
     * @param Settings $settings
     */
    public function __construct(
        Context $context,
        Image $imageHelper,
        UrlInterface $urlBuilder,
        Premium $premium,
        ScopeConfigInterface $scopeConfig,
        RemoteAddress $remoteAddress,
        Settings $settings
    ) {
        $this->imageHelper = $imageHelper;
        $this->urlBuilder = $urlBuilder;
        $this->premium = $premium;
        $this->scopeConfig = $scopeConfig;
        $this->remoteAddress = $remoteAddress;
        $this->settings = $settings;
        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        $params = $this->getRequest()->getPost();
        $locale =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            $params['store_id']
        );
        $language = str_replace('_', '-', $locale);
        $ipAddress = $this->remoteAddress->getRemoteAddress();

        try {
            // process gallery images
            $images = $this->uploadImages(
                $params['store_id'],
                $language,
                $ipAddress,
                $params['selected_images'] ?? [],
                $params['images'] ?? [],
                $this->settings->getSettings('rules')
            );
            $failedImages = $images['failed_images'] ?? [];
            $failedImagesWithoutThumbnail = $failedImages;

            // process thumbnail
            $thumbnailFailed = false;
            
            $result = [
                'is_premium' => $this->premium->isPremium(),
                'success' => empty($failedImages)
            ];

            if (!empty($failedImages)) {
                $selectedImagesCount = count($params['selected_images'] ?? []);
                $failedImagesCount = count($failedImages);

                if (isset($params['selected_fields']) && !empty($params['selected_fields'])) {
                    $result['proceedable'] = true;
                } else {
                    if ($selectedImagesCount > 0) {
                        $result['proceedable'] = $failedImagesCount !== $selectedImagesCount + (
                            $thumbnailFailed ? 1 : 0
                        );
                    }
                }

                $result['success_images'] = $images['success_images'] ?? [];
                $result['failed_images'] = $failedImages;
                $result['failed_images_without_thumbnail'] = $failedImagesWithoutThumbnail;
            }
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('writetextai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'is_premium' => $this->premium->isPremium(),
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Upload images to api
     *
     * @param int $storeId
     * @param string $language
     * @param string $ipAddress
     * @param array $selectedImages
     * @param array $gallery
     * @param array $rules
     *
     * @return array
     */
    private function uploadImages(
        $storeId,
        $language,
        $ipAddress,
        $selectedImages,
        $gallery,
        $rules
    ) {
        $failedImages = [];
        $successImages = [];

        if (empty($selectedImages)) {
            return [
                'failed_images' => $failedImages,
                'success_images' => $successImages
            ];
        }

        $selectedImagesChunks = array_chunk($selectedImages, $rules['maxImageAltTextPerRequest']);

        foreach ($selectedImagesChunks as $selectedImagesChunk) {
            foreach ($gallery as $image) {
                if (!in_array($image['id'], $selectedImagesChunk)) {
                    continue;
                }

                try {
                    $successImages[] = $this->imageHelper->getImageApiId(
                        $storeId,
                        $ipAddress,
                        $language,
                        $image['id'],
                        $image['orig_url']
                    );
                } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
                    // phpcs:ignore Magento2.Functions.DiscouragedFunction
                    $failedImages[] = $image['url'];
                } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
                    // phpcs:ignore Magento2.Functions.DiscouragedFunction
                    $failedImages[] = $image['url'];
                }
            }
        }

        return [
            'failed_images' => $failedImages,
            'success_images' => $successImages
        ];
    }
}
