<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Settings;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;
use WriteTextAI\WriteTextAI\Model\GenerateOptionsManager;
use WriteTextAI\WriteTextAI\Helper\Generate as GenerateHelper;
use WriteTextAI\WriteTextAI\ViewModel\UserFieldsSelected;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\Helper\ProductAttributes as ProductAttributesHelper;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldsHelper;

class Restore extends Action
{
    /**
     * @var UserFieldsSelectedManager
     */
    protected $userFieldsSelectedManager;

    /**
     * @var GenerateOptionsManager
     */
    protected $generateOptionsManager;

    /**
     * @var GenerateHelper
     */
    protected $generateHelper;

    /**
     * @var UserFieldsSelected
     */
    protected $userFieldsSelected;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var ProductAttributesHelper
     */
    protected $productAttributesHelper;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var FieldsHelper
     */
    protected $fieldsHelper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param UserFieldsSelectedManager $userFieldsSelectedManager
     * @param GenerateOptionsManager $generateOptionsManager
     * @param GenerateHelper $generateHelper
     * @param UserFieldsSelected $userFieldsSelected
     * @param AiProductManager $aiProductManager
     * @param ProductAttributesHelper $productAttributesHelper
     * @param ProductHelper $productHelper
     * @param FieldsHelper $fieldsHelper
     */
    public function __construct(
        Context $context,
        UserFieldsSelectedManager $userFieldsSelectedManager,
        GenerateOptionsManager $generateOptionsManager,
        GenerateHelper $generateHelper,
        UserFieldsSelected $userFieldsSelected,
        AiProductManager $aiProductManager,
        ProductAttributesHelper $productAttributesHelper,
        ProductHelper $productHelper,
        FieldsHelper $fieldsHelper
    ) {
        $this->userFieldsSelectedManager = $userFieldsSelectedManager;
        $this->generateOptionsManager = $generateOptionsManager;
        $this->generateHelper = $generateHelper;
        $this->userFieldsSelected = $userFieldsSelected;
        $this->aiProductManager = $aiProductManager;
        $this->productAttributesHelper = $productAttributesHelper;
        $this->productHelper = $productHelper;
        $this->fieldsHelper = $fieldsHelper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $this->userFieldsSelectedManager->deleteAll();

            $tones = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('tones'),
                $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_TONES
                )
            );

            $userStyles = $this->userFieldsSelected->getUserFieldsSelected(
                UserFieldsSelectedManager::ACTION_TYPE_STYLE
            );
            $styles = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('styles'),
                $userStyles ? [$userStyles] : []
            );

            $audiences = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('audience'),
                $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_AUDIENCES
                )
            );

            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');

            $this->resetProductAttributesSelected($productId, $storeId);

            $aiProduct = $this->aiProductManager->getProduct($productId, $storeId);
            $product = $this->productHelper->getProduct($productId, $storeId);
            $language = $this->fieldsHelper->getStoreLanguage($storeId);
            
            $result = [
                'success' => true,
                'data' => [
                    'custom_tone' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_TONE
                    ),
                    'custom_style' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_STYLE
                    ),
                    'guide_steps' => filter_var($this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_GUIDE_STEPS
                    ), FILTER_VALIDATE_BOOLEAN),
                    'selected_fields' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_SINGLE_EDITOR_FIELDS
                    ),
                    'highlighted' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHTED
                    ),
                    'highlight_pronouns' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHT_PRONOUNS
                    ),
                    'tones' => $tones,
                    'styles' => $styles,
                    'audience' => $audiences,
                    'product_attributes' => $this->productAttributesHelper->getProductAttributes($product),
                    'product_attributes_selected' => $this->productAttributesHelper
                        ->getProductAttributesSelected($aiProduct),
                    'gallery' => $this->userFieldsSelected->getUserFieldsSelected(
                        UserFieldsSelectedManager::ACTION_TYPE_GALLERY
                    ),
                    'images' => $this->productHelper->getImages($product, $storeId, $language)
                ]
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Reset product attributes selected
     *
     * @param int $productId
     * @param int $storeId
     * @return void
     */
    protected function resetProductAttributesSelected($productId, $storeId)
    {
        $this->aiProductManager->saveAttributesSelected(
            $productId,
            $storeId,
            ''
        );
    }
}
