<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

class Data extends Settings
{
    /**
     * Get API auth Url
     *
     * @return string
     */
    public function getApiAuthUrl()
    {
        return $this->getApiSettings('auth_url');
    }
    
    /**
     * Get API web Url
     *
     * @return string
     */
    public function getApiWebUrl()
    {
        return $this->getApiSettings('web_url');
    }
    
    /**
     * Get Signalr Url
     *
     * @return string
     */
    public function getSignalrUrl()
    {
        $signalRUrl =  $this->getApiSettings('signalr_url');
        $regionUrl = (string) $this->getApiSettings('region');
        if ($regionUrl) {
            $signalRUrl = sprintf($signalRUrl, $regionUrl);
        }
        return $signalRUrl;
    }

    /**
     * Get API region Url
     *
     * @return string
     */
    public function getRegion()
    {
        $region = (string) $this->getApiSettings('region');
        $regionUrl = $region;
        if (substr($regionUrl, 0, 4) !== "http") {
            $regionUrl = "https://{$region}/";
        }

        return $regionUrl;
    }

    /**
     * Get API client id
     *
     * @return string
     */
    public function getClientId()
    {
        return $this->getApiSettings('client_id');
    }

    /**
     * Get account token
     *
     * @return string
     */
    public function getAccountToken()
    {
        return $this->getApiSettings('account_token');
    }

    /**
     * Check if logger is enabled
     *
     * @return bool
     */
    public function isLoggerEnabled()
    {
        return (bool)$this->getApiSettings('enable_logger');
    }

    /**
     * Get tones
     *
     * @return string
     */
    public function getTones()
    {
        return $this->getGeneralSettings('tones');
    }

    /**
     * Get styles
     *
     * @return string
     */
    public function getStyles()
    {
        return $this->getGeneralSettings('styles');
    }

    /**
     * Get audiences
     *
     * @return string
     */
    public function getAudiences()
    {
        return $this->getGeneralSettings('audiences');
    }

    /**
     * Get selected countries
     *
     * @return array
     */
    public function getSelectedCountries()
    {
        $countries = $this->getCountrySettings('selected_countries');
        
        if ($countries) {
            return explode(',', $countries);
        }

        return [];
    }

    /**
     * Set account token
     *
     * @param string $token
     *
     * @return void
     */
    public function setAccountToken($token)
    {
        $this->setConfig(self::API_SETTINGS_CONFIG_PATH . 'account_token', $token);
    }

    /**
     * Set region
     *
     * @param string $region
     *
     * @return void
     */
    public function setRegion($region)
    {
        $this->setConfig(self::API_SETTINGS_CONFIG_PATH . 'region', $region);
    }

    /**
     * Check if custom open graph is enabled
     *
     * @return bool
     */
    public function getCustomOpenGraph()
    {
        return (bool)$this->getMappingSettings('custom_open_graph');
    }
    
    /**
     * Get data mapping fields
     *
     * @return array
     */
    public function getDataMappingFields()
    {
        return [
            'page_title'                => __('Meta title'),
            'page_description'          => __('Meta description'),
            'product_description'       => __('Description'),
            'short_product_description' => __('Short description'),
            'open_graph'                => __('Open Graph text')
        ];
    }

    /**
     * Get text type attributes
     *
     * @return array
     */
    public function getTextTypeAttributes()
    {
        return $this->textTypeAttributes->toOptionArray();
    }

    /**
     * Check if all text fields are mapped to an existing attribute
     *
     * @return boolean
     */
    public function isDataMapped()
    {
        if (empty($this->getMappingSettings())) {
            return false;
        }
        
        $attributes = array_column($this->getTextTypeAttributes(), 'value');
        foreach ($this->getMappingSettings() as $field => $attribute) {
            if ($field === 'open_graph' && !$this->getCustomOpenGraph()) {
                continue;
            }
            if (!$attribute || !in_array($attribute, $attributes)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Get not mapped field
     *
     * @return string
     */
    public function getNotMappedField()
    {
        // Set first field as not mapped field in case all field is not mapped
        $notMappedField = 'page_title';

        if (empty($this->getMappingSettings())) {
            return $this->getFieldLabel($notMappedField);
        }

        $attributes = array_column($this->getTextTypeAttributes(), 'value');
        foreach ($this->getMappingSettings() as $field => $attribute) {
            if ($field === 'open_graph' && !$this->getCustomOpenGraph()) {
                continue;
            }
            if (!$attribute || !in_array($attribute, $attributes)) {
                $notMappedField = $field;
            }
        }

        return $this->getFieldLabel($notMappedField);
    }

    /**
     * Get field label
     *
     * @param string $field
     * @return string
     */
    public function getFieldLabel($field)
    {
        switch ($field) {
            case 'page_title':
                return __('Meta title');
            case 'page_description':
                return __('Meta description');
            case 'product_description':
                return __('Description');
            case 'short_product_description':
                return __('Short description');
            case 'open_graph':
                return __('Open Graph text');
        }
    }

    /**
     * Check if final step is done
     *
     * @return boolean
     */
    public function isFinalStepDone()
    {
        $generalFields = $this->getGeneralSettings();
        if (empty($generalFields)) {
            return false;
        }
        
        $requiredFields = [
            'tones',
            'styles',
        ];

        foreach ($generalFields as $field => $value) {
            if (empty($value) && in_array($field, $requiredFields)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Check if setup is finalized
     *
     * @return bool
     */
    public function isSetupFinalized()
    {
        return $this->getSetupSettings('finalized');
    }

    /**
     * Get reference product collection
     *
     * @param int $storeId
     * @param int $productId
     * @param string $search
     * @param int $page
     * @param int $pageSize
     * @return array
     */
    public function getReferenceProductCollection(
        $storeId = 0,
        $productId = null,
        $search = '',
        $page = 1,
        $pageSize = 50
    ) {
        $collection = $this->productCollectionFactory->create();
        $collection->setStoreId($storeId);
        $collection->addFieldToSelect(['id', 'name']);

        if ($productId) {
            $collection->addFieldToFilter('entity_id', ['neq' => $productId]);
        }

        if ($search) {
            $collection->addFieldToFilter('name', ['like' => '%' . $search . '%']);
        }

        $attributes = $this->getMappingSettings();
        foreach ($attributes as $attribute) {
            $collection->addAttributeToFilter($attribute, ['nin' => [null, '']]);
        }

        $collection->addAttributeToSort('name', 'ASC');

        $total = $collection->getSize();

        $collection->getSelect()->limit($page * $pageSize);

        $result = [];
        foreach ($collection as $product) {
            $productDescription = $this->getAttributesValues($product, 'product_description');
            $shortProductDescription = $this->getAttributesValues($product, 'short_product_description');
            $productName = $this->catalogOutput->productAttribute(
                $product,
                $product->getName(),
                'name'
            );
            $result[] = [
                'value' => $product->getId(),
                'label' => $productName,
                'product_description' => $productDescription,
                'short_product_description' => $shortProductDescription,
            ];
        }

        return [
            'total' => $total,
            'items' => $result
        ];
    }

    /**
     * Get attributes values
     *
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @param string $field
     * @return string
     */
    public function getAttributesValues($product, $field)
    {
        $mapping = $this->getMappingSettings();
        $html = '';

        if (isset($mapping[$field])) {
            $attribute = $mapping[$field];
            $value = $product->getData($attribute);
            
            if ($value) {
                $html = $this->catalogOutput->productAttribute($product, $value, $attribute);
            }
        }

        return $html;
    }
}
