<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use WriteTextAI\WriteTextAI\Model\Config\Source\DisallowedHtml;
use Magento\Framework\Filter\FilterManager;

class Html extends AbstractHelper
{
    /**
     * @var FilterManager
     */
    protected $filterManager;

    /**
     * Constructor.
     * @param FilterManager $filterManager
     */
    public function __construct(
        FilterManager $filterManager
    ) {
        $this->filterManager = $filterManager;
    }

    /**
     * Remove disallowed html
     *
     * @param string $html
     * @return string
     */
    public function removeDisallowedHtml($html)
    {
        $html = $this->removeScriptAndStyleContent($html);
        $html = $this->removeDisallowedAttributes($html, DisallowedHtml::ALLOWED_ATTRIBUTES);
        $html = $this->removeDisallowedStyles($html, DisallowedHtml::ALLOWED_STYLES);

        $params = [
            'allowableTags' => DisallowedHtml::ALLOWED_TAGS,
            'escape' => false
        ];
        
        $html = $this->filterManager->stripTags($html, $params);

        return $html;
    }

    /**
     * Remove script and style content
     *
     * @param string $html
     * @return string
     */
    private function removeScriptAndStyleContent($html)
    {
        $dom = new \DOMDocument();
        // phpcs:disable Generic.PHP.NoSilencedErrors.Discouraged
        @$dom->loadHTML(utf8_decode("<div>{$html}</div>"), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
        $xpath = new \DOMXPath($dom);

        foreach ($xpath->query('//script') as $node) {
            $node->nodeValue = '';
        }

        foreach ($xpath->query('//style') as $node) {
            $node->nodeValue = '';
        }

        $result = "";
        foreach ($dom->documentElement->childNodes as $child) {
            $result .= $dom->saveHTML($child);
        }
        
        return $result;
    }

    /**
     * Remove disallowed attributes
     *
     * @param string $html
     * @param array $allowedAttributes
     * @return string
     */
    private function removeDisallowedAttributes($html, $allowedAttributes)
    {
        $dom = new \DOMDocument();
        // phpcs:disable Generic.PHP.NoSilencedErrors.Discouraged
        @$dom->loadHTML(utf8_decode("<div>{$html}</div>"), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
        $xpath = new \DOMXPath($dom);

        foreach ($xpath->query('//@*') as $node) {
            if (!isset($allowedAttributes[$node->parentNode->nodeName]) ||
                !in_array($node->nodeName, $allowedAttributes[$node->parentNode->nodeName])
            ) {
                $node->parentNode->removeAttribute($node->nodeName);
            }
        }

        $result = "";
        foreach ($dom->documentElement->childNodes as $child) {
            $result .= $dom->saveHTML($child);
        }
        
        return $result;
    }

    /**
     * Remove disallowed styles
     *
     * @param string $html
     * @param array $allowedStyles
     * @return string
     */
    private function removeDisallowedStyles($html, $allowedStyles)
    {
        $dom = new \DOMDocument();
        // phpcs:disable Generic.PHP.NoSilencedErrors.Discouraged
        @$dom->loadHTML(utf8_decode("<div>{$html}</div>"), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
        $xpath = new \DOMXPath($dom);

        foreach ($xpath->query('//@style') as $node) {
            $styles = explode(';', $node->nodeValue);
            $newStyles = [];

            foreach ($styles as $style) {
                $style = trim($style);

                if ($style) {
                    $style = explode(':', $style);
                    $style[0] = trim($style[0]);
                    $style[1] = trim($style[1]);

                    if (isset($allowedStyles[$node->parentNode->nodeName]) && (
                            in_array($style[0], $allowedStyles[$node->parentNode->nodeName]) ||
                            in_array('*', $allowedStyles[$node->parentNode->nodeName])
                        )
                    ) {
                        $newStyles[] = $style[0] . ': ' . $style[1];
                    }
                }
            }

            $node->nodeValue = implode('; ', $newStyles);
        }

        $result = "";
        foreach ($dom->documentElement->childNodes as $child) {
            $result .= $dom->saveHTML($child);
        }
        
        return $result;
    }
}
