<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\Exception\FileSystemException;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Helper\Image as ImageHelper;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Fields;

class Product extends AbstractHelper
{
    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var ImageHelper
     */
    protected $imageHelper;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Fields
     */
    protected $fields;

    /**
     * Constructor
     *
     * @param GeneratedText $generatedText
     * @param ProductRepositoryInterface $productRepository
     * @param StoreManagerInterface $storeManager
     * @param ImageHelper $imageHelper
     * @param ApiManager $apiManager
     * @param Fields $fields
     */
    public function __construct(
        GeneratedText $generatedText,
        ProductRepositoryInterface $productRepository,
        StoreManagerInterface $storeManager,
        ImageHelper $imageHelper,
        ApiManager $apiManager,
        Fields $fields
    ) {
        $this->generatedText = $generatedText;
        $this->productRepository = $productRepository;
        $this->storeManager = $storeManager;
        $this->imageHelper = $imageHelper;
        $this->apiManager = $apiManager;
        $this->fields = $fields;
    }

    /**
     * Get reviewed
     *
     * @param int $productId
     * @param int $storeId
     * @param array $text
     * @return bool
     */
    public function getReviewed($productId, $storeId, $text = [])
    {
        $text = !empty($text) ? $text : $this->generatedText->getGenerated($productId, $storeId);

        if (!isset($text['records']) || empty($text['records'])) {
            return false;
        }

        $fieldsReviewed = [];
        foreach ($text['records'] as $record) {
            if ($record['recordId'] == $productId) {
                foreach ($record['stores'] as $store) {
                    if ($store['storeId'] == $storeId && (
                        $store['field'] !== 'open graph text' || (
                            $store['field'] === 'open graph text' &&
                            $this->fields->getCustomOpenGraph()
                        )
                    )) {
                        if (isset($store['texts'][0]['history'][0]['reviewed'])) {
                            $fieldsReviewed[] = $store['texts'][0]['history'][0]['reviewed'];
                        } else {
                            $fieldsReviewed[] = false;
                        }
                    }
                }
            }
        }

        return !empty($fieldsReviewed) && !in_array(false, $fieldsReviewed);
    }

    /**
     * Get product
     *
     * @param int $productId
     * @param int $storeId
     * @return \Magento\Catalog\Api\Data\ProductInterface
     */
    public function getProduct($productId, $storeId)
    {
        return $this->productRepository->getById($productId, false, $storeId);
    }

    /**
     * Get frontend store id
     *
     * @param int $storeId
     * @return int
     */
    public function getFrontendStoreId($storeId)
    {
        if ($storeId == \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store::DEFAULT_STORE_ID) {
            return $this->storeManager->getDefaultStoreView()->getId();
        }

        return $storeId;
    }

    /**
     * Get store scope
     *
     * @param int $storeId
     * @return string
     */
    public function getStoreScope($storeId)
    {
        if ($storeId == \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store::DEFAULT_STORE_ID) {
            $scope = __('All store views');
        } else {
            $store = $this->storeManager->getStore($storeId);
            $scope = $store->getName();
        }

        return $scope;
    }
    
    /**
     * Get website by store id
     *
     * @param int $storeId
     * @return \Magento\Store\Api\Data\WebsiteInterface
     */
    public function getWebsiteByStoreId($storeId)
    {
        $store = $this->storeManager->getStore($storeId);
        $website = $store->getWebsite();

        return $website;
    }

    /**
     * Get images
     *
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @param int $storeId
     * @param string $language
     * @return array
     */
    public function getImages($product, $storeId, $language)
    {
        $images = [];
        $mediaGallery = $product->getMediaGalleryImages()->setOrder('position', 'ASC');
        $imageIds = [];
        $imageLimit = 10;
        $imageCount = 0;

        if (!empty($mediaGallery)) {
            foreach ($mediaGallery as $mediaGalleryEntry) {
                if ($mediaGalleryEntry->getDisabled()) {
                    continue;
                }

                if ($imageCount >= $imageLimit) {
                    break;
                }

                $imageFileThumbnail = $this->imageHelper->init($product, 'product_listing_thumbnail')
                    ->setImageFile($mediaGalleryEntry->getFile());
                $url = $imageFileThumbnail->getUrl();

                $origUrl = $this->imageHelper->init($product, 'product_listing_thumbnail_preview')
                    ->setImageFile($mediaGalleryEntry->getFile())
                    ->getUrl();

                /**
                 * Do not include the image file if it cannot be processed.
                 */
                try {
                    $imageFileThumbnail->getOriginalWidth();
                } catch (\InvalidArgumentException | FileSystemException $e) {
                    continue;
                }

                $imageIds[] = [
                    'id' => $mediaGalleryEntry->getId(),
                    'url' => $url,
                    'alt' => $mediaGalleryEntry->getLabel() ?? '',
                    'orig_url' => $origUrl
                ];
                
                $imageCount++;
            }
        }

        $chunks = array_chunk($imageIds, 20);

        foreach ($chunks as $chunk) {
            $ids = array_column($chunk, 'id');
            try {
                $apiImages = $this->apiManager->getImageList([
                    "language" => $language,
                    "images" => $ids,
                    "storeId" => $storeId,
                    "includeUpdateHistory" => "true"
                ]);

                foreach ($chunk as $image) {
                    $wtaAltText = '';
                    $wtaPublish = false;
                    $wtaTextId = null;
                    $filteredImages = array_filter(
                        $apiImages['images'],
                        function ($filteredImage) use ($image) {
                            return $filteredImage['imageId']== $image['id'];
                        }
                    );
                    $apiImage = reset($filteredImages);

                    if ($apiImage) {
                        $wtaAltText = $apiImage['altText']['value'] ?? '';
                        $wtaPublish = $apiImage['altText']['history'][0]['publish'] ?? false;
                        $wtaTextId = $apiImage['altText']['id'] ?? null;
                    }

                    $images[] = [
                        'url' => $image['url'],
                        'orig_url' => $image['orig_url'],
                        'alt' => $image['alt'],
                        'id' => $image['id'],
                        'original_writetext_alt' => $wtaAltText,
                        'writetext_alt' => $wtaAltText,
                        'writetext_alt_id' => $wtaTextId,
                        'status' => [
                            'generated' => $wtaAltText !== '',
                            'transferred' => $wtaPublish
                        ]
                    ];
                }
            } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedIf
                if ($e->getCode() === 404) {
                    // skip images not yet generated
                }
            }
        }

        return $images;
    }
}
