<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model\Api;

use Magento\Framework\HTTP\Client\CurlFactory as CurlClientFactory;
use Magento\Framework\HTTP\Adapter\CurlFactory as CurlAdapterFactory;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Logger\Logger;
use WriteTextAI\WriteTextAI\Model\UserWebTokenManager;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Module\ModuleListInterface;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Model\Api\StatusCodeValidator;
use WriteTextAI\WriteTextAI\Model\Api\VersionValidator;
use WriteTextAI\WriteTextAI\Exception\UnauthorizedException;
use WriteTextAI\WriteTextAI\Exception\VersionException;
use Magento\Framework\Message\ManagerInterface as MessageManager;

/**
 * @api
 */
class CurlBuilder
{
    public const MODULE_NAME = 'WriteTextAI_WriteTextAI';

    /**
     * @var CurlClientFactory
     */
    protected $curlClientFactory;

    /**
     * @var CurlAdapterFactory
     */
    protected $curlAdapterFactory;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * @var ModuleListInterface
     */
    protected $moduleList;

    /**
     * @var UserWebTokenManager
     */
    protected $userWebTokenManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var StatusCodeValidator
     */
    protected $statusCodeValidator;

    /**
     * @var VersionValidator
     */
    protected $versionValidator;
    
    /**
     * @var MessageManager
     */
    protected $messageManager;

    /**
     * Constructor
     *
     * @param CurlClientFactory $curlClientFactory
     * @param CurlAdapterFactory $curlAdapterFactory
     * @param Data $helper
     * @param Logger $logger
     * @param UserWebTokenManager $userWebTokenManager
     * @param ProductMetadataInterface $productMetadata
     * @param ModuleListInterface $moduleList
     * @param ApiSession $apiSession
     * @param StatusCodeValidator $statusCodeValidator
     * @param VersionValidator $versionValidator
     * @param MessageManager $messageManager
     */
    public function __construct(
        CurlClientFactory $curlClientFactory,
        CurlAdapterFactory $curlAdapterFactory,
        Data $helper,
        Logger $logger,
        UserWebTokenManager $userWebTokenManager,
        ProductMetadataInterface $productMetadata,
        ModuleListInterface $moduleList,
        ApiSession $apiSession,
        StatusCodeValidator $statusCodeValidator,
        VersionValidator $versionValidator,
        MessageManager $messageManager
    ) {
        $this->curlClientFactory = $curlClientFactory;
        $this->curlAdapterFactory = $curlAdapterFactory;
        $this->helper = $helper;
        $this->logger = $logger;
        $this->userWebTokenManager = $userWebTokenManager;
        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;
        $this->apiSession = $apiSession;
        $this->statusCodeValidator = $statusCodeValidator;
        $this->versionValidator = $versionValidator;
        $this->messageManager = $messageManager;
    }

    /**
     * Get api auth url
     *
     * @return string
     */
    public function getApiAuthUrl()
    {
        return $this->helper->getApiAuthUrl();
    }

    /**
     * Get api web url
     *
     * @return string
     */
    public function getApiWebUrl()
    {
        return $this->helper->getApiWebUrl();
    }

    /**
     * Get saved region
     *
     * @return string
     */
    private function getSavedRegion()
    {
        return $this->helper->getRegion();
    }

    /**
     * Get client id
     *
     * @return string
     */
    public function getClientId()
    {
        return $this->helper->getClientId();
    }

    /**
     * Ping region
     *
     * @param string $region
     * @return string
     */
    public function pingRegion($region)
    {
        try {
            $curl = $this->curlAdapterFactory->create();
            // headers
            $headers = ["Cache-Control" => "no-cache"];
            // body
            $regionUrl = $region;
            if (substr($regionUrl, 0, 4) !== "http") {
                $regionUrl = "https://{$region}";
            }
            $apiUrl = $regionUrl . '/text/ping';
            // log request
            $this->logger->logRequest($apiUrl);
            // send request
            $curl->write(
                'GET',
                $apiUrl,
                '1.1',
                $headers
            );
            // get response
            $response = $curl->getInfo(CURLINFO_TOTAL_TIME);
            // log response
            $this->logger->logResponse($curl->getInfo(CURLINFO_HTTP_CODE), $response);
            // close request
            $curl->close();
            // return response
            return $response;
        } catch (\Exception $e) {
            $this->logger->logResponseErrors($e->getMessage());
        }
    }

    /**
     * Get regions
     *
     * @return array
     */
    public function getRegions()
    {
        try {
            $curl = $this->curlClientFactory->create();
            // body
            $apiUrl = $this->getApiWebUrl() . 'web/regions';
            // log request
            $this->logger->logRequest($apiUrl);
            // send request
            $curl->get($apiUrl);
            // get response
            $response = json_decode($curl->getBody(), true);
            // log response
            $this->logger->logResponse($curl->getStatus(), $response);
            // return response
            return $response;
        } catch (\Exception $e) {
            $this->logger->logResponseErrors($e->getMessage());
        }
    }
    
    /**
     * Get user web token
     *
     * @param bool $renew
     * @return string
     */
    public function getUserWebToken($renew = false)
    {
        $user = $this->apiSession->getCurrentUser();
        if ($user) {
            $email = $user->getEmail();
            $userId = $user->getId();

            if ($renew) {
                $this->userWebTokenManager->deleteUserWebToken($userId);
            }
        
            $webToken = $this->userWebTokenManager->getUserWebToken($userId);
            if (!$webToken) {
                $webToken = $this->getWebToken($email, $userId);
            }
        } else {
            $webToken = $this->getWebToken();
        }

        return $webToken;
    }

    /**
     * Get api response
     *
     * @param string $apiPath
     * @param array $body
     * @param string $type
     * @param string $acceptLanguage
     *
     * @return mixed
     */
    public function getApiResponse(
        string $apiPath,
        $body = [],
        $type = 'get',
        $acceptLanguage = 'en-US'
    ) {
        try {
            $curl = $this->curlClientFactory->create();
            // get web token
            $webToken = $this->getUserWebToken();
            // headers
            $platformVersion = $this->productMetadata->getVersion();
            $moduleVersion = $this->getModuleVersion();
            $this->addHeaders(
                $curl,
                [
                    "Content-Type" => "application/json",
                    "Accept-Language" => $acceptLanguage,
                    "Authorization" => "Bearer " . $webToken,
                    "WriteTextAI-PlatformVersion" => $platformVersion,
                    "WriteTextAI-PluginVersion" => $moduleVersion,
                    "WriteTextAI-PHPVersion" => phpversion(),
                    "Expect" => ""
                ]
            );
            // body
            $apiUrl = $this->getSavedRegion() . $apiPath;
            if ($type === 'get' && !empty($body)) {
                $apiUrl .= '?' . http_build_query($body);
            }
            // log request
            $this->logger->logRequest($apiUrl, $body);
            // send request
            $curl->$type($apiUrl, $body);
            // check response
            if ($curl->getStatus() === StatusCodeValidator::UNAUTHORIZED) {
                $this->handleUnauthorizedRequest($curl, $apiUrl, $body, $type);
            }
            // validate response
            $this->versionValidator->validate($curl, $moduleVersion);
            $this->statusCodeValidator->validate($curl);
            // get response
            $response = json_decode($curl->getBody(), true);
            // log response
            $this->logger->logResponse($curl->getStatus(), $response);
            // return response
            return $response;
        } catch (VersionException $e) {
            $this->messageManager->getMessages(true);
            $this->messageManager->addError($e->getMessage());

            throw $e;
        }
    }

    /**
     * Get web token
     *
     * @param string $email
     * @param string $userId
     *
     * @return string
     */
    public function getWebToken($email = null, $userId = null)
    {
        try {
            $curl = $this->curlClientFactory->create();
            // get account token
            $accountToken = $this->helper->getAccountToken();
            // get connect token
            $connectToken = $this->getConnectToken($accountToken);
            // headers
            $platformVersion = $this->productMetadata->getVersion();
            $moduleVersion = $this->getModuleVersion();
            $this->addHeaders(
                $curl,
                [
                    "Cache-Control" => "no-cache",
                    "Authorization" => "Bearer " . $connectToken,
                    "WriteTextAI-PlatformVersion" => $platformVersion,
                    "WriteTextAI-PluginVersion" => $moduleVersion,
                    "WriteTextAI-PHPVersion" => phpversion()
                ]
            );
            // body
            $apiUrl = $this->getApiWebUrl() . 'web/token';
            if ($email) {
                $params = ['email' => $email];
                $apiUrl .= '?' . http_build_query($params);
            }
            // log request
            $this->logger->logRequest($apiUrl);
            // send request
            $curl->get($apiUrl);
            // validate response
            $this->statusCodeValidator->validate($curl);
            // get response
            $response = json_decode($curl->getBody(), true);
            // log response
            $this->logger->logResponse($curl->getStatus(), $response);
            // save web token
            if ($userId) {
                $this->userWebTokenManager->setUserWebToken($userId, $response['access_token']);
            }
            // return web token
            return $response['access_token'];
        } catch (\Exception $e) {
            throw new UnauthorizedException(__($e->getMessage()));
        }
    }

    /**
     * Get connect token
     *
     * @param string $accountToken
     * @return string
     */
    public function getConnectToken($accountToken)
    {
        try {
            $curl = $this->curlClientFactory->create();
            // headers
            $platformVersion = $this->productMetadata->getVersion();
            $moduleVersion = $this->getModuleVersion();
            $this->addHeaders(
                $curl,
                [
                    "Cache-Control" => "no-cache",
                    "Content-Type" => "application/x-www-form-urlencoded",
                    "WriteTextAI-PlatformVersion" => $platformVersion,
                    "WriteTextAI-PluginVersion" => $moduleVersion,
                    "WriteTextAI-PHPVersion" => phpversion()
                ]
            );
            // body
            $apiUrl = $this->getApiAuthUrl() . 'connect/token';
            $body = [
                'grant_type' => 'refresh_token',
                'client_id' => $this->getClientId(),
                'refresh_token' => $accountToken
            ];
            // log request
            $this->logger->logRequest($apiUrl, $body);
            // send request
            $curl->post($apiUrl, $body);
            // validate response
            $this->statusCodeValidator->validate($curl);
            // get response
            $response = json_decode($curl->getBody(), true);
            // log response
            $this->logger->logResponse($curl->getStatus(), $response);
            // return connect token
            return $response['access_token'];
        } catch (\Exception $e) {
            throw new UnauthorizedException(__($e->getMessage()));
        }
    }

    /**
     * Add headers
     *
     * @param MagentoCurl $curl
     * @param array $params
     * @return void
     */
    protected function addHeaders($curl, $params)
    {
        foreach ($params as $key => $value) {
            $curl->addHeader($key, $value);
        }
    }

    /**
     * Handle unauthorized request
     *
     * @param MagentoCurl $curl
     * @param string $apiUrl
     * @param array $body
     * @param string $type
     * @return void
     */
    protected function handleUnauthorizedRequest($curl, $apiUrl, $body, $type)
    {
        // try again with new token
        $renew = true;
        $webToken = $this->getUserWebToken($renew);
        $curl->addHeader("Authorization", "Bearer " . $webToken);
        $curl->$type($apiUrl, $body);
    }

    /**
     * Get module version
     *
     * @return string
     */
    protected function getModuleVersion()
    {
        return $this->moduleList->getOne(self::MODULE_NAME)['setup_version'];
    }
}
