<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model\Api;

use Magento\Framework\HTTP\Client\Curl as MagentoCurl;
use Magento\Framework\Serialize\Serializer\Json;
use WriteTextAI\WriteTextAI\Exception\ApiException;
use WriteTextAI\WriteTextAI\Exception\UnauthorizedException;
use WriteTextAI\WriteTextAI\Logger\Logger;

class StatusCodeValidator
{
    const UNAUTHORIZED = 401;

    /**
     * @var Json
     */
    protected $json;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * StatusCodeValidator constructor.
     *
     * @param Json $json
     * @param Logger $logger
     */
    public function __construct(
        Json $json,
        Logger $logger
    ) {
        $this->json = $json;
        $this->logger = $logger;
    }

    /**
     * Validate the status code of the response
     *
     * @param MagentoCurl $curl
     * @return bool
     */
    public function validate(MagentoCurl $curl): bool
    {
        if (!$this->isSuccess($curl->getStatus())
            && !$this->isContinue($curl->getStatus())
        ) {
            $errMsg = $this->getErrorMessage($curl->getBody());
            $status = $curl->getStatus();

            if ($status === self::UNAUTHORIZED) {
                $exceptionMessage = sprintf('[%s] %s', $status, $errMsg);
                $this->logger->logResponseErrors($exceptionMessage);
                throw new UnauthorizedException(__($exceptionMessage));
            } elseif ($this->isClientError($status)) {
                $exceptionMessage = sprintf('[%s] Request issue: %s', $status, $errMsg);
                $this->handleApiErrorCodes($curl->getBody(), $exceptionMessage, $status);
            } elseif ($this->isServerError($status)) {
                $exceptionMessage = sprintf('[%s] Server error: %s', $status, $errMsg);
            } else {
                $exceptionMessage = sprintf('[%s] There was an issue processing the request: %s', $status, $errMsg);
            }

            $this->logger->logResponseErrors($exceptionMessage);
            throw new ApiException(__($errMsg), null, $status);
        }

        return true;
    }

    /**
     * Handle API error codes
     *
     * @param string $curlBody
     * @param string $message
     * @param int $status
     * @return bool
     */
    protected function handleApiErrorCodes($curlBody, $message, $status)
    {
        $body = $this->json->unserialize($curlBody);

        if (!isset($body['code'])) {
            return;
        }

        switch ($body['code']) {
            case 28:
                throw new \WriteTextAI\WriteTextAI\Exception\ImageDownloadFailed(__($message), null, $status);
            case 29:
                throw new \WriteTextAI\WriteTextAI\Exception\ImageUnsupported(__($message), null, $status);
            case 30:
                throw new \WriteTextAI\WriteTextAI\Exception\ImageInvalid(__($message), null, $status);
            default:
                return;
        }
    }

    /**
     * Check if the response is ongoing
     *
     * @param $code
     * @return bool
     */
    protected function isContinue($code): bool
    {
        return (100 <= $code && 200 > $code);
    }
    /**
     * Check if the response is successful
     *
     * @param $code
     * @return bool
     */
    protected function isSuccess($code): bool
    {
        return (200 <= $code && 300 > $code);
    }

    /**
     * Check if the response is a client error
     *
     * @param $code
     * @return bool
     */
    protected function isClientError($code): bool
    {
        return ($code >= 400 && $code < 500);
    }

    /**
     * Check if the response is a server error
     *
     * @param $code
     * @return bool
     */
    protected function isServerError($code): bool
    {
        return $code >= 500;
    }

    /**
     * Get the error message from the response
     *
     * @param string $curlBody
     * @return string
     */
    protected function getErrorMessage($curlBody = '')
    {
        try {
            $unserializedMessage = $this->json->unserialize($curlBody);
            $error = $unserializedMessage['errors'] ?? $unserializedMessage['error'] ?? $curlBody;

            if (is_array($error) || is_object($error)) {
                $error = json_encode($error);
            }

            return $error;
        } catch (\InvalidArgumentException $e) {
            return $curlBody;
        }
    }
}
