<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldsHelper;
use WriteTextAI\WriteTextAI\Model\Api\CurlBuilder;

/**
 * @api
 */
class ApiManager
{
    /**
     * @var FieldsHelper
     */
    protected $fieldsHelper;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var CurlBuilder
     */
    protected $curlBuilder;

    /**
     * Constructor
     *
     * @param RemoteAddress $remoteAddress
     * @param FieldsHelper $fieldsHelper
     * @param CurlBuilder $curlBuilder
     */
    public function __construct(
        RemoteAddress $remoteAddress,
        FieldsHelper $fieldsHelper,
        CurlBuilder $curlBuilder
    ) {
        $this->remoteAddress = $remoteAddress;
        $this->fieldsHelper = $fieldsHelper;
        $this->curlBuilder = $curlBuilder;
    }

    /**
     * Get tones
     *
     * @return array
     */
    public function getTones()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Tones') ?? [];
    }

    /**
     * Get styles
     *
     * @return array
     */
    public function getStyles()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Styles') ?? [];
    }

    /**
     * Get audiences
     *
     * @return array
     */
    public function getAudiences()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Audiences') ?? [];
    }

    /**
     * Get credit
     *
     * @return array
     */
    public function getCredit()
    {
        return $this->curlBuilder->getApiResponse('text/Credit') ?? [];
    }

    /**
     * Get ads
     *
     * @return array
     */
    public function getAds()
    {
        return $this->curlBuilder->getApiResponse('web/ad/SquareMagento') ?? [];
    }

    /**
     * Get credits
     *
     * @return array
     */
    public function getCredits()
    {
        $response = $this->curlBuilder->getApiResponse('text/Generate/Credits');

        $propertyMapping = [
            "page title" => "page_title",
            "page description" => "page_description",
            "product description" => "product_description",
            "excerpt" => "short_product_description",
            "open graph text" => "open_graph"
        ];

        $this->fieldsHelper->changePropertyNames($response, $propertyMapping);

        return $response ?? [];
    }

    /**
     * Get formal languages
     *
     * @return array
     */
    public function getFormalLanguageSupport()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/FormalLanguageSupport');
    }

    /**
     * Upload image
     *
     * @param array $body
     * @return array
     */
    public function uploadImage($body)
    {
        return $this->curlBuilder->getApiResponse('text/Image', $body, 'post');
    }

    /**
     * Get image
     *
     * @param array $body
     * @return array
     */
    public function getImage($body)
    {
        return $this->curlBuilder->getApiResponse('text/Image', $body);
    }

    /**
     * Get image list
     *
     * @param array $body
     * @return array
     */
    public function getImageList($body)
    {
        return $this->curlBuilder->getApiResponse('text/Image/List', $body);
    }

    /**
     * Generate alt text
     *
     * @param array $body
     * @return array
     */
    public function generateAltText($body)
    {
        return $this->curlBuilder->getApiResponse('text/Image/GenerateAltText', $body, 'post');
    }

    /**
     * Update alt text
     *
     * @param array $body
     * @return array
     */
    public function updateAltText($body)
    {
        return $this->curlBuilder->getApiResponse('text/Image/altText', $body, 'post');
    }

    /**
     * Get formal languages
     *
     * @param string $acceptLanguage
     *
     * @return array
     */
    public function getFormalLanguages($acceptLanguage = 'en-US')
    {
        return $this->curlBuilder->getApiResponse(
            'text/Generate/FormalLanguages',
            [],
            'get',
            $acceptLanguage
        );
    }

    /**
     * Get disallowed combinations
     *
     * @return array
     */
    public function getDisallowedCombinations()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/DisallowedCombinations');
    }

    /**
     * Get rules
     *
     * @return array
     */
    public function getRules()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Rules');
    }

    /**
     * Get reviews
     *
     * @param array $params
     * @return string
     */
    public function getReviews($params = [])
    {
        return $this->curlBuilder->getApiResponse('text/Review', $params);
    }

    /**
     * Update review status
     *
     * @param array $body
     * @return string
     */
    public function updateReview($body)
    {
        return $this->curlBuilder->getApiResponse('text/Review', $body, 'post');
    }

    /**
     * Generate
     *
     * @param array $body
     * @return string
     */
    public function generate($body)
    {
        return $this->curlBuilder->getApiResponse('text/Generate', $body, 'post');
    }

    /**
     * Cancel generate
     *
     * @param string $id
     * @return string
     */
    public function cancelGenerate($id)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Bulk/' . $id . '/cancel', [], 'post');
    }

    /**
     * Retry generate
     *
     * @param string $id
     * @return string
     */
    public function retryGenerate($id)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Bulk/' . $id . '/retry', [], 'post');
    }

    /**
     * Get generate status
     *
     * @return string
     */
    public function getGenerateStatus()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Bulk');
    }

    /**
     * Get generate status
     *
     * @param string $id
     *
     * @return string
     */
    public function getBulkRequestById($id)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Bulk/' . $id);
    }

    /**
     * Get generate status by id
     *
     * @param string $id
     * @return string
     */
    public function getGenerateStatusById($id)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Bulk/' . $id);
    }

    /**
     * Get generated text
     *
     * @param array $body
     * @return string
     */
    public function getGenerated($body)
    {
        return $this->curlBuilder->getApiResponse('text/Generate', $body);
    }

    /**
     * Get generated status
     *
     * @param array $body
     * @return string
     */
    public function getGeneratedStatus($body)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Status', $body);
    }

    /**
     * Get review status
     *
     * @param array $body
     * @return string
     */
    public function filterReviewStatus($body)
    {
        return $this->curlBuilder->getApiResponse('text/Review/Status', $body);
    }

    /**
     * Get locations
     *
     * @return array
     */
    public function getLocations()
    {
        $acceptLanguage = 'en-US';

        $response = $this->curlBuilder->getApiResponse('text/KeywordAnalysis/Locations', [], 'get', $acceptLanguage);

        return $response['result'] ?? [];
    }

    /**
     * GetKeywordIdeas
     *
     * @param string $storeId
     * @param string $recordId
     * @param array $keywords
     * @param string $location
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param bool $competitionSort
     * @param bool $volumeSort
     * @param bool $keywordSort
     * @param int $page
     * @param int $pageSize
     * @param bool $refresh
     * @param bool $nogenerate
     * @param string $ipAddress
     *
     * @return array
     */
    public function getKeywordIdeas(
        $storeId,
        $recordId,
        array $keywords,
        $location,
        array $filterByCompetition = [
            "LOW",
            "MEDIUM",
            "HIGH"
        ],
        $filterBySearchVolumeMinimum = 0,
        $filterBySearchVolumeMaximum = null,
        $competitionSort = '',
        $volumeSort = '',
        $keywordSort = '',
        $page = 1,
        $pageSize = 10,
        $refresh = true,
        $nogenerate = false
    ) {

        $sorting = [];
        if ($competitionSort != '') {
            $sorting[] = [
                "field" => "competition_index",
                "ascending" => $competitionSort == 'true' ? true : false
            ];
        }

        if ($volumeSort != '') {
            $sorting[] = [
                "field" => "search_volume",
                "ascending" => $volumeSort == 'true' ? true : false
            ];
        }

        if ($keywordSort != '') {
            $sorting[] = [
                "field" => "keyword",
                "ascending" => $keywordSort == 'true' ? true : false
            ];
        }

        $ipAddress = $this->remoteAddress->getRemoteAddress();
        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            'ipAddress' => $ipAddress,
            "text" => $keywords,
            "location_code" => $location,
            "refresh" => $refresh,
            "nogenerate" => $nogenerate,
            "page" => $page,
            "pageSize" => $pageSize,
            "filterByCompetition" => $filterByCompetition,
            "filterBySearchVolumeMinimum" => $filterBySearchVolumeMinimum,
            "sorting" => $sorting

        ];

        if ($filterBySearchVolumeMaximum) {
            $body['filterBySearchVolumeMaximum'] = $filterBySearchVolumeMaximum;
        }

        $acceptLanguage =  $this->fieldsHelper->getStoreLanguage($storeId);

        try {
            $response = $this->curlBuilder->getApiResponse(
                'text/KeywordAnalysis/Ideas',
                json_encode($body),
                'post',
                $acceptLanguage
            );
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Get semantic keywords
     *
     * @param string $storeId
     * @param string $recordId
     * @param array $keywords
     * @param string $clear
     *
     * @return string
     */
    public function getSemanticKeywords(
        $storeId,
        $recordId,
        $keywords,
        $clear = 'false'
    ) {
        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "text" => $keywords,
            "clearAllText" => $clear
        ];

        $acceptLanguage =  $this->fieldsHelper->getStoreLanguage($storeId);

        return $this->curlBuilder->getApiResponse('text/Suggestion/Keywords', $body, 'get', $acceptLanguage);
    }

    /**
     * Select semantic keyword
     *
     * @param string $storeId
     * @param string $recordId
     * @param array $selected
     * @param string $text
     * @return string
     */
    public function selectSemanticKeyword(
        $storeId,
        $recordId,
        array $selected,
        $text
    ) {
        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "text" => $text,
            "selected" => $selected
        ];

        $acceptLanguage =  $this->fieldsHelper->getStoreLanguage($storeId);

        return $this->curlBuilder->getApiResponse(
            'text/Suggestion/Keywords',
            json_encode($body),
            'post',
            $acceptLanguage
        );
    }

    /**
     * Get suggested audience
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $keywords
     * @param string $clear
     * @return string
     */
    public function getSuggestedAudience(
        $storeId,
        $recordId,
        $keywords,
        $clear = 'false'
    ) {
        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "text" => $keywords,
            "clearAllText" => $clear
        ];

        $acceptLanguage =  $this->fieldsHelper->getStoreLanguage($storeId);

        return $this->curlBuilder->getApiResponse('text/Suggestion/Audience', $body, 'get', $acceptLanguage);
    }

    /**
     * Select audience
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $keywords
     * @param array $selected
     * @return string
     */
    public function selectAudience(
        $storeId,
        $recordId,
        $keywords,
        $selected
    ) {
        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "text" => $keywords,
            "selected" => $selected
        ];

        $acceptLanguage =  $this->fieldsHelper->getStoreLanguage($storeId);

        return $this->curlBuilder->getApiResponse(
            'text/Suggestion/Audience',
            json_encode($body),
            'post',
            $acceptLanguage
        );
    }

    /**
     * Get history
     *
     * @param array $params
     */
    public function getHistory($params = [])
    {
        return $this->curlBuilder->getApiResponse('text/Generate/History', $params);
    }

    /**
     * Save
     *
     * @param array $body
     * @return string
     */
    public function save($body)
    {
        return $this->curlBuilder->getApiResponse('text/Generate/text', json_encode($body), 'post');
    }

    /**
     * Get ETags
     *
     * @return array
     */
    public function getETags()
    {
        return $this->curlBuilder->getApiResponse('text/Generate/Etags');
    }

    /**
     * Get region
     *
     * @return string
     */
    public function getRegion()
    {
        $regions = $this->curlBuilder->getRegions();
        $fastest = [];

        foreach ($regions as $region) {
            $time = $this->curlBuilder->pingRegion($region);
            if (isset($fastest['time']) ? $time < $fastest['time'] : true) {
                $fastest = [
                    'region' => $region,
                    'time' => $time
                ];
            }
        }

        return $fastest['region'];
    }
}
