<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\BulkGenerateRequestsFactory;
use WriteTextAI\WriteTextAI\Api\BulkGenerateRequestsRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\ApiManager;

class BulkGenerateRequestsManager
{
    /**
     * @var BulkGenerateRequestsFactory
     */
    protected $BulkGenerateRequestsFactory;

    /**
     * @var BulkGenerateRequestsRepositoryInterface
     */
    protected $BulkGenerateRequestsRepository;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * Constructor
     *
     * @param BulkGenerateRequestsFactory $BulkGenerateRequestsFactory
     * @param BulkGenerateRequestsRepositoryInterface $BulkGenerateRequestsRepository
     * @param ApiManager $apiManager
     */
    public function __construct(
        BulkGenerateRequestsFactory $BulkGenerateRequestsFactory,
        BulkGenerateRequestsRepositoryInterface $BulkGenerateRequestsRepository,
        ApiManager $apiManager
    ) {
        $this->BulkGenerateRequestsFactory = $BulkGenerateRequestsFactory;
        $this->BulkGenerateRequestsRepository = $BulkGenerateRequestsRepository;
        $this->apiManager = $apiManager;
    }

    /**
     * Set bulk request
     *
     * @param string $requestId
     * @param string $user
     * @return void
     */
    public function setBulkRequest($requestId, $user)
    {
        $BulkGenerateRequests = $this->BulkGenerateRequestsFactory->create();

        $existing = $BulkGenerateRequests->getCollection()
            ->addFieldToFilter('user', $user)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setRequestId($requestId);
            $BulkGenerateRequests = $existing;
        } else {
            // Record not found, create a new one
            $BulkGenerateRequests->setData([
                "user" => $user,
                "request_id" => $requestId
            ]);
        }

        $this->BulkGenerateRequestsRepository->save($BulkGenerateRequests);
    }

    /**
     * Delete bulk request
     *
     * @param string $requestId
     * @return void
     */
    public function delete($requestId)
    {
        $BulkGenerateRequests = $this->BulkGenerateRequestsFactory->create();

        $existing = $BulkGenerateRequests->getCollection()
            ->addFieldToFilter('request_id', $requestId)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->BulkGenerateRequestsRepository->delete($existing);
        }
    }

    /**
     * Delete bulk request by user
     *
     * @param string $user
     * @return void
     */
    public function deleteByUser($user)
    {
        $BulkGenerateRequests = $this->BulkGenerateRequestsFactory->create();

        $existing = $BulkGenerateRequests->getCollection()
            ->addFieldToFilter('user', $user)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->BulkGenerateRequestsRepository->delete($existing);
        }
    }

    /**
     * Delete all bulk requests
     *
     * @return void
     */
    public function deleteAll()
    {
        $BulkGenerateRequests = $this->BulkGenerateRequestsFactory->create();
        $BulkGenerateRequests->getCollection()->walk('delete');
    }

    /**
     * Get bulk requests
     *
     * @return bool
     */
    public function getBulkRequests()
    {
        $BulkGenerateRequests = $this->BulkGenerateRequestsFactory->create();
        $BulkGenerateRequests = $BulkGenerateRequests->getCollection();

        $requests = [];
        foreach ($BulkGenerateRequests as $BulkGenerateRequest) {
            try {
                $apiResponse = $this->apiManager->getBulkRequestById($BulkGenerateRequest->getRequestId());
            } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                if ($e->getCode() === 404) {
                    $apiResponse = [
                        'status' => 'Completed',
                        'completed' => 1,
                        'total' => 1,
                        'queuedIds' => [],
                        'completedIds' => [],
                        'userName' => $BulkGenerateRequest->getUser(),
                        'estimatedEndTime' => null
                    ];
                } else {
                    throw $e;
                }
            }

            $remaining = $this->calculateRemainingTime($apiResponse['estimatedEndTime']);
            $message = $this->getMessage($apiResponse['status'], $remaining['remaining'], $remaining['is_minutes']);
            $requests[] = [
                'id' => $BulkGenerateRequest->getRequestId(),
                'message' => $message,
                'progress' => sprintf(
                    __('%s / %s product/s'),
                    $apiResponse['completed'],
                    $apiResponse['total']
                ),
                'percent' => floor($apiResponse['completed'] / $apiResponse['total'] * 100),
                'status' => $apiResponse['status'],
                'queuedIds' => $apiResponse['queuedIds'],
                'completedIds' => $apiResponse['completedIds'],
                'user' => $apiResponse['userName']
            ];
        }

        return $requests;
    }

    /**
     * Calculate remaining time
     *
     * @param string $estimatedEndTime
     * @return array
     */
    protected function calculateRemainingTime($estimatedEndTime)
    {
        $remaining = 0;
        $isMinutes = false;
        $result = [];
        $currentDatetime;
        $interval;

        if ($estimatedEndTime !== null) {
            $estimatedEndTime = new \DateTime($estimatedEndTime);
            $currentDatetime = new \DateTime();
            $interval = $estimatedEndTime->getTimestamp() - $currentDatetime->getTimestamp();
            // Convert seconds to minutes if it's more than or equal to 60 seconds
            $remaining = ceil($interval / 60);

            if ($remaining >= 60) {
                // Convert minutes to hours if it's more than or equal to 60 minutes
                $remaining = ceil($remaining / 60);
                $isMinutes = true;
            }
        }

        $result['remaining'] = $remaining;
        $result['is_minutes'] = $isMinutes;

        return $result;
    }

    /**
     * Get message
     *
     * @param string $requestStatus
     * @param int $remaining
     * @param bool $isMinutes
     *
     * @return {String}
     */
    protected function getMessage($requestStatus, $remaining, $isMinutes)
    {
        $message = '';
        $text = '';

        switch ($requestStatus) {
            case 'Pending':
                $message = __('Generating text...');
                break;
            case 'Running':
                if ($isMinutes) {
                    $text = __('Generating text... (Estimated time remaining %s minute/s)');
                } else {
                    $text = __('Generating text... (Estimated time remaining %s second/s)');
                }
                $message = sprintf($text, $remaining);
                break;
            case 'Completed':
                $message = __('WriteText.ai is done generating text for your selected products.');
                break;
            case 'Cancelling':
                $message = __('Cancelling...');
                break;
            case 'Cancelled':
                $message = __('This process has been cancelled.');
                break;
            case 'TimedOut':
                $message = __('WriteText.ai is currently experiencing a high volume of requests and has timed out.');
                break;
        }

        return $message;
    }
}
