<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\BulkTransferRequestsFactory;
use WriteTextAI\WriteTextAI\Api\BulkTransferRequestsRepositoryInterface;

class BulkTransferRequestsManager
{
    /**
     * @var BulkTransferRequestsFactory
     */
    protected $BulkTransferRequestsFactory;

    /**
     * @var BulkTransferRequestsRepositoryInterface
     */
    protected $BulkTransferRequestsRepository;

    /**
     * Constructor
     *
     * @param BulkTransferRequestsFactory $BulkTransferRequestsFactory
     * @param BulkTransferRequestsRepositoryInterface $BulkTransferRequestsRepository
     */
    public function __construct(
        BulkTransferRequestsFactory $BulkTransferRequestsFactory,
        BulkTransferRequestsRepositoryInterface $BulkTransferRequestsRepository
    ) {
        $this->BulkTransferRequestsFactory = $BulkTransferRequestsFactory;
        $this->BulkTransferRequestsRepository = $BulkTransferRequestsRepository;
    }

    /**
     * Save bulk request
     *
     * @param string $user
     * @param array $stores
     * @param string $storeFilter
     * @param array $fields
     * @param array $queueIds
     * @param array $completedIds
     * @return void
     */
    public function saveBulkRequest(
        $user,
        $stores,
        $storeFilter,
        $fields,
        $queueIds = [],
        $completedIds = []
    ) {
        $queueIds = !empty($queueIds) ? implode(',', $queueIds) : '';
        $completedIds = !empty($completedIds) ? implode(',', $completedIds) : '';
        $stores = implode(',', $stores);
        $fields = implode(',', $fields);

        $BulkTransferRequests = $this->BulkTransferRequestsFactory->create();

        $existing = $BulkTransferRequests->getCollection()
            ->addFieldToFilter('user', $user)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setUser($user);
            $existing->setQueueIds($queueIds);
            $existing->setCompletedIds($completedIds);
            $existing->setStoresSelected($stores);
            $existing->setStoreFilter($storeFilter);
            $existing->setFieldsSelected($fields);
            $BulkTransferRequests = $existing;
        } else {
            // Record not found, create a new one
            $BulkTransferRequests->setData([
                "user" => $user,
                "queue_ids" => $queueIds,
                "completed_ids" => $completedIds,
                "stores_selected" => $stores,
                "store_filter" => $storeFilter,
                "fields_selected" => $fields
            ]);
        }

        $this->BulkTransferRequestsRepository->save($BulkTransferRequests);
    }

    /**
     * Delete bulk request
     *
     * @param string $user
     * @return void
     */
    public function delete($user)
    {
        $BulkTransferRequests = $this->BulkTransferRequestsFactory->create();

        $existing = $BulkTransferRequests->getCollection()
            ->addFieldToFilter('user', $user)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->BulkTransferRequestsRepository->delete($existing);
        }
    }

    /**
     * Delete all bulk requests
     *
     * @return void
     */
    public function deleteAll()
    {
        $BulkTransferRequests = $this->BulkTransferRequestsFactory->create();
        $BulkTransferRequests->getCollection()->walk('delete');
    }

    /**
     * Get bulk requests
     *
     * @return bool
     */
    public function getBulkRequests()
    {
        $BulkTransferRequests = $this->BulkTransferRequestsFactory->create();
        $BulkTransferRequests = $BulkTransferRequests->getCollection();

        $requests = [];
        foreach ($BulkTransferRequests as $BulkTransferRequest) {
            $requests[] = [
                'queue_ids' => $BulkTransferRequest->getQueueIds() ?
                    explode(',', $BulkTransferRequest->getQueueIds()) : [],
                'completed_ids' => $BulkTransferRequest->getCompletedIds() ?
                    explode(',', $BulkTransferRequest->getCompletedIds()) : [],
                'user' => $BulkTransferRequest->getUser(),
                'stores' => explode(',', $BulkTransferRequest->getStoresSelected()),
                'store_filter' => $BulkTransferRequest->getStoreFilter(),
                'fields' => explode(',', $BulkTransferRequest->getFieldsSelected())
            ];
        }

        return $requests;
    }
}
