/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/transfer/settings',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'Magento_Ui/js/modal/confirm',
    'mage/template',
    'text!Magento_Ui/templates/grid/cells/thumbnail/preview.html',
    'Magento_Ui/js/modal/modal'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    premium,
    gallery,
    textfieldsUtils,
    product,
    transferSettings,
    prompt,
    totalCredits,
    confirm,
    mageTemplate,
    thumbnailPreviewTemplate
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            selectedImages: [],
            selectAllImages: false,
            images: [],
            highlighted: true,
            selectedImagesNotPremium: [],
            selectAllImagesNotPremium: false
        },

        isSelectedChange: false,
        isSelectAllChange: false,
        isImagesChange: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;
            
            gallery.images.subscribe(function (images) {
                if (!self.isImagesChange) {
                    self.images(images);
                }
            });

            keywords.highlighted.subscribe(function (highlighted) {
                self.highlighted(highlighted);
            });

            gallery.selectAllImages.subscribe(function (selectAllImages) {
                self.selectAllImages(selectAllImages);
            });

            gallery.selectedImages.subscribe(function (selectedImages) {
                self.isSelectedChange = true;
                self.selectedImages(selectedImages);
                self.isSelectedChange = false;
            });

            self.selectAllImages.subscribe(function (selectAllImages) {
                var images = selectAllImages ? [ ...self.images().map(image => image.id) ] : [];
                
                if (!self.isSelectAllChange) {
                    self.isSelectedChange = true;
                    self.selectedImages(images);
                    gallery.selectedImages(images);
                    if (selectAllImages !== gallery.selectAllImages()) {
                        gallery.selectAllImages(selectAllImages);
                    }
                    self.isSelectedChange = false;
                }

            });

            self.selectedImages.subscribe(function (selectedImages) {
                var selectAllImages = selectedImages.length === self.images().length;

                if (!self.isSelectedChange) {
                    self.isSelectAllChange = true;
                    self.selectAllImages(selectAllImages);
                    gallery.selectAllImages(selectAllImages);
                    gallery.selectedImages(selectedImages);
                    self.isSelectAllChange = false;
                
                    self.savePreferences('gallery_images', selectedImages);
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'selectedImages',
                    'selectAllImages',
                    'images',
                    'highlighted',
                    'selectedImagesNotPremium',
                    'selectAllImagesNotPremium'
                ]);

                this.isImgAltTextFieldEnabled = function (image_id) {
                    return ko.computed(function () {
                        var returnStatus = false;
                        gallery.images().some(function (image) {
                            if (image.id === image_id) {
                                returnStatus = image.writetext_alt !== '' && image.status.generated;
                                return returnStatus;
                            }
                        });
                        return returnStatus;
                    }, this);
                };

                this.isPremium = ko.computed(function () {
                    return premium.isPremium();
                });

            return this;
        },

        /**
         * Preview image.
         */
        preview: function (image) {
            var modalHtml = mageTemplate(
                thumbnailPreviewTemplate,
                {
                    src: image,
                    alt: '',
                    link: '',
                    linkText: ''
                    }
            ),
                previewPopup = $('<div/>').html(modalHtml);

            previewPopup.modal({
                title: '',
                innerScroll: true,
                modalClass: '_image-box',
                buttons: []
            }).trigger('openModal');
        },

        /**
         * Save user preferences
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value.length <= 0) {
                value = '';
            }

            if (field === 'gallery_images') {
                value = {
                    images: value,
                    product_id: product.productId(),
                };
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Process click functionality of select all
         * Important to always return true
         * in order for checked binding to continue working
         *
         * @returns {boolean}
         */
        processClick: function () {
            var value = gallery.selectAllImages();
            self.savePreferences('gallery', value);
            return true;
        },


        /**
         * Get char count.
         *
         * @param {String} html
         * @param {Number} limit
         * @returns {Number}
         */
        getCharCount: function (html, limit = null) {
            return textfieldsUtils.getCharCount(html, limit);
        },

        /**
         * Get word count.
         *
         * @param {String} html
         * @returns {Number}
         */
        getWordCount: function (html) {
            return textfieldsUtils.getWordCount(html);
        },

        /**
         * Update image.
         *
         * @param {Object} image
         * @returns {void}
         */
        updateImage: function (data, event, image) {
            var images = self.images();
            
            images.forEach(function (img) {
                if (img.id === image.id) {
                    img.writetext_alt = event.target.value;
                }
            });

            self.isImagesChange = true;
            gallery.images([]);
            gallery.images(images);
            self.isImagesChange = false;
            $('#writetext-alt-' + image.id).trigger('focus');
        },

        /**
         * Check if transfer is enabled.
         *
         * @param {Object} data
         * @returns {Boolean}
         */
        isTransferEnabled: function (data) {
            return data.writetext_alt !== '' && data.status.generated;
        },

        /**
         * Transfer image to Magento
         *
         * @param {Object} image
         */
        transfer: function (image) {
            confirm({
                title: $.mage.__('Transfer selected to Magento'),
                content: self.confirmContent,
                modalClass:
                    'writetextai-modal bulk-transfer-popup transfer edit',
                actions: {
                    confirm: function () {
                        self._transfer(image);
                    },
                    cancel: function () {
                        return;
                    }
                }
            });
        },

        /**
         * Transfer image to Magento
         *
         * @param {Object} image
         */
        _transfer: function (image) {
            $.ajax({
                url: self.transferUrl,
                type: 'POST',
                data: {
                    image: image,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    selected_stores: transferSettings.selectedStores()
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showGalleryLoader: true,
                success: function (response) {
                    if (typeof response.is_premium !== undefined) {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (response.success) {
                        gallery.images(response.images);
                    } else {
                        console.error(response.message);
                    }
                }
            });
        }
    });
});
