/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'Magento_Ui/js/form/element/ui-select',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/reference-products',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/reference-product',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'mage/translate',
    'Magento_Ui/js/block-loader',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'Magento_Ui/js/lib/key-codes'
], function (
    $,
    ko,
    Select,
    _,
    referenceProducts,
    genRefProduct,
    product,
    $t,
    blockLoader,
    premium,
    keyCodes
) {
    'use strict';

    /**
     * @var {Select} self
     */
    var self;

    blockLoader();

    return Select.extend({
        defaults: {
            filterInputSelector:
                '.reference-product .admin__action-multiselect-search',
            template: 'ui/form/field',
            multiple: false,
            filterOptions: true,
            selectedPlaceholders: {
                defaultPlaceholder: $t('Select a product...')
            },
            filterPlaceholder: $t('Search products'),
            labelVisible: false,
            productId: '',
            storeId: '',
            selectedOptionLabel: '',
            page: 1,
            pageSize: 50,
            totalSize: 0,
            lazyLoad: true,
            isLoading: false,
            itemsQuantity: '0 ' + $t('options'),
            listens: {
                filterInputValue: ''
            }
        },

        typingTimer: null,
        productsLoaded: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            referenceProducts.referenceProducts.subscribe(
                function (options) {
                    self.options(options);
                    self.resetOptions();
                }
            );

            referenceProducts.referenceProductTotal.subscribe(
                function (referenceProductTotal) {
                    self.totalSize(referenceProductTotal);
                    self.itemsQuantity(
                        referenceProductTotal + ' ' + self.quantityPlaceholder
                    );
                }
            );

            referenceProducts.referenceProductChecked.subscribe(
                function (referenceProductChecked) {
                    if (referenceProductChecked && !self.productsLoaded) {
                        self.filterOptionsList(false, true, false);
                        self.productsLoaded = true;
                    }
                }
            );

            self.value.subscribe(function (value) {
                genRefProduct.referenceProduct(value);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'selectedOptionLabel',
                'page',
                'pageSize',
                'totalSize',
                'isLoading'
            ]);

            this.isAllowed = ko.computed(function () {
                var isChecked = referenceProducts.referenceProductChecked(),
                    isPremium = premium.isPremium();
                
                return isChecked && isPremium;
            }, this);

            return this;
        },

        /**
         * Reset value
         */
        resetValue: function () {
            self.value([]);
        },

        /**
         * Reset products loaded
         */
        resetProductsLoaded: function () {
            self.productsLoaded = false;
            self.filterInputValue('');
        },

        /**
         * Reset options and filter input value
         */
        resetOptions: function () {
            self.page(1);
        },

        /**
         * Load more options on scroll down
         * @param {Object} data
         * @param {Event} event
         */
        onScrollDown: function (data, event) {
            var clientHight =
                    event.target.scrollTop + event.target.clientHeight,
                scrollHeight = event.target.scrollHeight;

            if (this.totalSize() <= this.options().length && !this.lazyLoad) {
                return;
            }

            if (clientHight > scrollHeight - this.deviation) {
                self.page(self.page() + 1);
                self.filterOptionsAction(true);
            }
        },
        
        /** @inheritdoc */
        filterOptionsKeydown: function (data, event) {
            var key = keyCodes[event.keyCode];

            if (key === 'enterKey') {
                self.filterOptionsList(false, false, false);
            } else {
                self.filterOptionsList(false, false, true);
            }
            
            return true;
        },

        /** @inheritdoc */
        keyDownHandlers: function () {
            return {
                escapeKey: this.escapeKeyHandler,
                spaceKey: this.enterKeyHandler,
                pageUpKey: this.pageUpKeyHandler,
                pageDownKey: this.pageDownKeyHandler
            };
        },

        /**
         * Filtered options list by value from filter options list
         */
        filterOptionsList: function (showMore = false, clear = false, delay = true) {
            clearTimeout(self.typingTimer);
            if (delay) {
                self.typingTimer = setTimeout(function () {
                    self.filterOptionsAction(showMore, clear);
                }, 2000);
            } else {
                self.filterOptionsAction(showMore, clear);
            }
        },

        /**
         * Filtered options list by value from filter options list
         *
         * @param {Boolean} showMore
         * @returns {void}
         */
        filterOptionsAction: function (showMore) {
            var url = self.filterReferencesUrl,
                productId = self.productId(),
                storeId = self.storeId(),
                page = self.page(),
                pageSize = self.pageSize(),
                search = self.filterInputValue().trim().toLowerCase();

            self.isLoading(true);

            if (!showMore) {
                self.page(1);
            }

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    search: search,
                    product_id: productId,
                    store_id: storeId,
                    page: page,
                    page_size: pageSize
                },
                dataType: 'json',
                success: function (response) {
                    if (response.success) {
                        self.options(response.products);
                        self.totalSize(response.total);
                        referenceProducts.referenceProducts(response.products);
                        referenceProducts.referenceProductTotal(response.total);
                        self.itemsQuantity(
                            response.total + ' ' + self.quantityPlaceholder
                        );
                        setTimeout(function () {
                            $(self.filterInputSelector).trigger('focus');
                        });
                    }
                }
            }).always(function () {
                self.isLoading(false);
            });
        },

        /**
         * Set caption
         */
        setCaption: function () {
            var length,
                caption = '';

            if (!_.isArray(this.value()) && this.value()) {
                length = 1;
            } else if (this.value()) {
                length = this.value().length;
            } else {
                this.value([]);
                length = 0;
            }
            this.warn(caption);

            if (
                this.isDisplayMissingValuePlaceholder &&
                length &&
                !this.getSelected().length
            ) {
                caption = this.missingValuePlaceholder.replace(
                    '%s',
                    this.value()
                );
                this.placeholder(self.selectedOptionLabel());
                this.warn(caption);

                return this.placeholder();
            }

            if (length > 1) {
                this.placeholder(
                    length + ' ' + this.selectedPlaceholders.lotPlaceholders
                );
            } else if (length && this.getSelected().length) {
                /**
                 * Get label not from cached options but from the selected option
                 */
                this.placeholder(self.selectedOptionLabel());
            } else {
                this.placeholder(this.selectedPlaceholders.defaultPlaceholder);
            }

            return this.placeholder();
        },

        /**
         * Show more options
         */
        showMore: function () {
            self.page(self.page() + 1);
            self.filterOptionsList(true, false, false);
        },

        /**
         * Toggle activity list element
         *
         * @param {Object} data - selected option data
         * @returns {Object} Chainable
         */
        toggleOptionSelected: function (data) {
            self.selectedOptionLabel(data.label);
            genRefProduct.referenceProductData(data);
            return this._super(data);
        },

        /**
         * Get path to current option
         *
         * @param {Object} data - option data
         * @returns {String} path
         */
        getPath: function (data) {
            if (typeof data.path !== 'undefined') {
                return this._super(data);
            }
        }
    });
});
