/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'ko',
    'underscore',
    'mage/translate',
    'domReady!'
], function (
    ko,
    _,
    $t
) {
    'use strict';

    /**
     * @var {Array} statuses
     * @var {String} tempId
     * @var {String} transferTempId
     */
    var statuses = ko.observableArray([]),
        tempId = 'temp',
        transferTempId = 'transferTemp',
        opened = ko.observable(false);

    /**
     * @param {String} format
     * @returns {String}
     */
    function sprintf(format)
    {
        var args = Array.prototype.slice.call(arguments, 1);

        return format.replace(/%s/g, function () {
            return args.shift();
        });
    }

    /**
     * @param {Array} array1
     * @param {Array} array2
     * @param {Array} array3
     * @returns {Boolean}
     */
    function checkArraysIntersection(array1, array2 = [], array3)
    {
        for (let i = 0; i < array1.length; i++) {
            if (array3.includes(array1[i])) {
                return true;
            }
        }

        for (let i = 0; i < array2.length; i++) {
            if (array3.includes(array2[i])) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param {String} requestStatus
     * @param {Number} remaining
     * @param {Boolean} isMinutes
     * @returns {String}
     */
    function getMessage(requestStatus, remaining, isMinutes)
    {
        var message = '',
            text = '';

        switch (requestStatus) {
            case 'Pending':
                message = $t('Generating text...');
                break;
            case 'Running':
                if (isMinutes) {
                    text = $t('Generating text... (Estimated time remaining %s minute/s)');
                } else {
                    text = $t('Generating text... (Estimated time remaining %s second/s)');
                }
                message = sprintf(text, remaining);
                break;
            case 'Completed':
                message = $t('WriteText.ai is done generating text for your selected products.');
                break;
            case 'Cancelling':
                message = $t('Cancelling...');
                break;
            case 'Cancelled':
                message = $t('This process has been cancelled.');
                break;
            case 'TimedOut':
                message = $t('WriteText.ai is currently experiencing a high volume of requests and has timed out.');
                break;
        }

        return message;
    }

    /**
     * @param {String} estimatedEndTime
     * @returns {Object}
     */
    function calculateRemainingTime(estimatedEndTime)
    {
        var remaining = 0,
            isMinutes = false,
            result = {},
            currentDatetime,
            interval;

        if (estimatedEndTime !== null) {
            estimatedEndTime = new Date(estimatedEndTime);
            currentDatetime = new Date();
            interval = estimatedEndTime.getTime() - currentDatetime.getTime();
            remaining = Math.ceil(interval / 1000); /* Convert milliseconds to seconds */

            if (remaining >= 60) {
                remaining = Math.ceil(remaining / 60); /* Convert seconds to minutes if it's more than or equal to 60 seconds */
                isMinutes = true;
            }
        }

        result.remaining = remaining;
        result.is_minutes = isMinutes;

        return result;
    }

    /**
     * @param {Object} notification
     * @param {Array} requestQueuedIds
     * @param {Array} requestCompletedIds
     * @param {String} requestId
     * @returns {void}
     */
    function addNotification(
        notification,
        requestQueuedIds,
        requestCompletedIds,
        requestId
    ) {
        var noStatusMatched = true;

        for (const status of statuses()) {
            if (
                status.id === tempId &&
                checkArraysIntersection(
                    requestQueuedIds,
                    requestCompletedIds,
                    status.queuedIds
                ) &&
                status.user === notification.user
            ) {
                statuses.replace(status, notification);
                noStatusMatched = false;
                break;
            }
            if (status.id === requestId) {
                notification.user = status.user;
                statuses.replace(status, notification);
                noStatusMatched = false;
                break;
            }
        }

        if (noStatusMatched) {
            const index = statuses().findIndex(status => status.user === notification.user);

            if (index !== -1) {
                statuses.splice(index, 1, notification);
            } else {
                statuses.push(notification);
            }
        }
    }

    /**
     * @param {Array} productIds
     * @param {Object} currentUser
     * @returns {void}
     */
    function addNewGenerateStatus(productIds, currentUser, data = {})
    {
        var initialLength = productIds.length;
        if (data) {
            initialLength = data.excludeMode ? data.total : initialLength;
        }
        var generateStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: tempId,
                message: $t('Generating text...'),
                progress: sprintf($t('%s / %s product/s'), 0, initialLength),
                user: currentUser.email,
                percent: 0,
                status: 'Running',
                queuedIds: productIds,
                completedIds: []
        };

        if (generateStatus) {
            statuses.replace(generateStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * @param {Array} productIds
     * @param {Object} currentUser
     * @returns {void}
     */
    function addNewTransferStatus(productIds, currentUser)
    {
        var transferStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: transferTempId,
                message: $t('Transferring text...'),
                progress: sprintf($t('%s / %s product/s'), 0, productIds.length),
                user: currentUser.email,
                percent: 0,
                status: 'Running',
                transfer: true,
                queuedIds: productIds,
                completedIds: []
        };

        if (transferStatus) {
            statuses.replace(transferStatus, newStatus);
        } else {
            statuses.push(newStatus);
        }
        opened(true);
    }

    /**
     * @param {Array} productIds
     * @param {Object} currentUser
     * @param {Array} queueIds
     * @param {Array} completedIds
     * @param {Object} continueParams
     * @returns {void}
     */
    function updateTransferStatus(productIds, currentUser, queueIds, completedIds, continueParams = {})
    {
        var transferStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser;
        }),
            message = $t('Done transferring text for your selected products.'),
            progress = sprintf($t('%s / %s product/s'), completedIds.length, productIds.length),
            percent = Math.floor(completedIds.length / productIds.length * 100),
            status = 'Completed',
            _queuedIds = [],
            _completedIds = [...productIds];

        if (completedIds.length !== productIds.length) {
            if (queueIds.length == 0) {
                message = $t('Cancelled');
                status = 'Completed';
                _queuedIds = [...queueIds];
                _completedIds = [...completedIds];
            } else {
                message = $t('Transferring text...');
                status = 'Running';
                _queuedIds = [...queueIds];
                _completedIds = [...completedIds];
            }
        }

        var newStatus = {
            id: transferTempId,
            message: message,
            progress: progress,
            user: currentUser,
            percent: percent,
            status: status,
            transfer: true,
            queuedIds: _queuedIds,
            completedIds: _completedIds,
            continueParams: continueParams
        };

        if (transferStatus) {
            statuses.replace(transferStatus, newStatus);
        } else {
            statuses.push(newStatus);
        }
    }

    /**
     * @param {Object} messageEntry
     * @returns {void}
     */
    function processNotification(messageEntry)
    {
        var requestId = messageEntry.encodedMsg.id,
            requestStatus = messageEntry.encodedMsg.status,
            requestEstimatedEndTime = messageEntry.encodedMsg.estimatedEndTime,
            requestQueuedIds = messageEntry.encodedMsg.queuedIds,
            requestCompletedIds = messageEntry.encodedMsg.completedIds,
            requestCompleted = messageEntry.encodedMsg.completed,
            requestTotal = messageEntry.encodedMsg.total,
            requestUserName = messageEntry.encodedMsg.userName,
            remaining = {},
            message = '',
            requestData = {};

        remaining = calculateRemainingTime(requestEstimatedEndTime);
        message = getMessage(requestStatus, remaining.remaining, remaining.is_minutes);
        requestData = {
            id: requestId,
            message: message,
            progress: sprintf(
                $t('%s / %s product/s'),
                requestCompleted,
                requestTotal
            ),
        percent: Math.floor(requestCompleted / requestTotal * 100),
        status: requestStatus,
        queuedIds: requestQueuedIds,
        completedIds: requestCompletedIds,
        user: requestUserName
        };

        addNotification(
            requestData,
            requestQueuedIds,
            requestCompletedIds,
            requestId
        );
    }

    /**
     * @param {Object} messageEntry
     * @returns {void}
     */
    function addStreamStatus(messageEntry)
    {
        processNotification(messageEntry);
    }

    return {
        statuses: statuses,
        tempId: tempId,
        transferTempId: transferTempId,
        addNewGenerateStatus: addNewGenerateStatus,
        addStreamStatus: addStreamStatus,
        addNewTransferStatus: addNewTransferStatus,
        updateTransferStatus: updateTransferStatus,
        opened: opened
    };
});
