/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/reference-products',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/reference-product',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields'
], function (
    $,
    premium,
    prompt,
    textfields,
    settings,
    gallery,
    referenceProducts,
    generateSettings,
    genRefProduct,
    textfieldsUtils
) {
    'use strict';

    var _rules = {},
        _credits = {},
        _selector = '';

    /**
     * Get meta title credits
     *
     * @param {string} field
     * @param {boolean} rewrite
     * @returns {number}
     */
    function getPageTitleCredits(field = 'page_title', rewrite = false)
    {
        if (rewrite && !textfields.mgPageTitle()) {
            return 0;
        }

        return _credits.generation[field];
    }

    /**
     * Get meta description credits
     *
     * @param {string} field
     * @param {boolean} rewrite
     * @returns {number}
     */
    function getPageDescriptionCredits(field = 'page_description', rewrite = false)
    {
        if (rewrite && !textfields.mgPageDescription()) {
            return 0;
        }

        return _credits.generation[field];
    }

    /**
     * Get description credits
     *
     * @param {string} field
     * @param {boolean} rewrite
     * @param {boolean} bulk
     * @returns {number}
     */
    function getProductDescriptionCredits(field = 'product_description', rewrite = false, bulk = false)
    {
        var referenceProductChecked = bulk ? prompt.referenceProductChecked() : referenceProducts.referenceProductChecked(),
            generation = _credits.generation[field],
            generationTier = _credits.generationTier[field],
            targetLength = bulk ? prompt.descMax() : generateSettings.descMax(),
            productDescription = textfields.mgProductDescription() ?? '',
            productDescriptionCharCount = textfieldsUtils.getCharCount(productDescription, null, false),
            referenceProduct = bulk ? prompt.selectedReferenceProduct() : genRefProduct.referenceProductData(),
            referenceProductCharCount = textfieldsUtils.getCharCount(referenceProduct.product_description),
            charCount = referenceProductChecked ? referenceProductCharCount : productDescriptionCharCount,
            charCountTierLevel = getTierLevel(charCount),
            targetLengthTierLevel = Math.ceil(targetLength / _credits.wordsPerCredit),
            tierLevel = targetLengthTierLevel;

        var html = `
            target length: ${targetLength},
            target length tier level: ${targetLengthTierLevel},
        `;

        if (rewrite && !textfields.mgProductDescription()) {
            return 0;
        }
            
        if ((referenceProductChecked || rewrite) && charCount !== undefined) {
            tierLevel = Math.max(charCountTierLevel, targetLengthTierLevel);
            html += `
            reference/rewrite char count: ${charCount},
            char tier level: ${charCountTierLevel},
            `;
        }

        if (rewrite) {
            $(_selector + `_rewrite .product_description`).html(html + `
            generation: ${generation},
            generation tier: ${generationTier},
            highest tier level: ${tierLevel},
            credits: ${generation + (generationTier * tierLevel)}
            `);
        } else {
            $(_selector + ` .product_description`).html(html + `
            generation: ${generation},
            generation tier: ${generationTier},
            highest tier level: ${tierLevel},
            credits: ${generation + (generationTier * tierLevel)}
            `);
        }

        return generation + (generationTier * tierLevel);
    }

    /**
     * Get short description credits
     *
     * @param {string} field
     * @param {boolean} rewrite
     * @param {boolean} bulk
     * @returns {number}
     */
    function getProductExcerptCredits(field = 'short_product_description', rewrite = false, bulk = false)
    {
        var referenceProductChecked = bulk ? prompt.referenceProductChecked() : referenceProducts.referenceProductChecked(),
            generation = _credits.generation[field],
            generationTier = _credits.generationTier[field],
            targetLength = bulk ? prompt.excMax() : generateSettings.excMax(),
            productExcerpt = textfields.mgProductShortDescription() ?? '',
            productExcerptCharCount = textfieldsUtils.getCharCount(productExcerpt, null, false),
            referenceProduct = bulk ? prompt.selectedReferenceProduct() : genRefProduct.referenceProductData(),
            referenceProductCharCount = textfieldsUtils.getCharCount(referenceProduct.short_product_description),
            charCount = referenceProductChecked ? referenceProductCharCount : productExcerptCharCount,
            charCountTierLevel = getTierLevel(charCount),
            targetLengthTierLevel = Math.ceil(targetLength / _credits.wordsPerCredit),
            tierLevel = targetLengthTierLevel;

        var html = `
            target length: ${targetLength},
            target length tier level: ${targetLengthTierLevel},
        `;

        if (rewrite && !textfields.mgProductShortDescription()) {
            return 0;
        }
            
        if ((referenceProductChecked || rewrite) && charCount !== undefined) {
            tierLevel = Math.max(charCountTierLevel, targetLengthTierLevel);
            html += `
            reference/rewrite char count: ${charCount},
            char tier level: ${charCountTierLevel},
            `;
        }

        if (rewrite) {
            $(_selector + `_rewrite .short_product_description`).html(html + `
            generation: ${generation},
            generation tier: ${generationTier},
            highest tier level: ${tierLevel},
            credits: ${generation + (generationTier * tierLevel)}
            `);
        } else {
            $(_selector + ` .short_product_description`).html(html + `
            generation: ${generation},
            generation tier: ${generationTier},
            highest tier level: ${tierLevel},
            credits: ${generation + (generationTier * tierLevel)}
            `);
        }

        return generation + (generationTier * tierLevel);
    }

    /**
     * Get open graph credits
     *
     * @param {string} field
     * @param {boolean} rewrite
     * @returns {number}
     */
    function getOpenGraphCredits(field = 'open_graph', rewrite = false)
    {
        if (rewrite && !textfields.mgOpenGraph()) {
            return 0;
        }

        return _credits.generation[field];
    }

    /**
     * Get credit level chart
     *
     * @returns {array}
     */
    function getCreditsChart()
    {
        var chart = [],
            credit = 0,
            additionalReferenceTextLength = _rules.additionalReferenceTextLength,
            maxOutputWords = _rules.maxOutputWords,
            wordsPerCredit = _credits.wordsPerCredit,
            charCount = _rules.maxReferenceTextLength;

        for (let index = wordsPerCredit; index <= maxOutputWords; index += wordsPerCredit) {
            credit++;
            chart.push({
                charCount: charCount,
                credits: credit
            });
            charCount += additionalReferenceTextLength;
        }

        return chart;
    }

    /**
     * Get tier level
     *
     * @param {number} charCount
     * @returns {number}
     */
    function getTierLevel(charCount)
    {
        var highestCredit = 0,
            chart = getCreditsChart();

        for (const level of chart) {
            highestCredit = level.credits;

            if (level.charCount >= charCount) {
                break;
            }
        }

        return highestCredit;
    }

    /**
     * Get generate credits
     *
     * @param {object} rules
     * @param {boolean} rewrite
     * @param {boolean} bulk
     * @param {object} bulkCredits
     * @returns {number}
     */
    function getCredits(rules, rewrite = false, bulk = false, bulkCredits = {})
    {
        var total = 0,
            selectedFields = bulk ? prompt.selectedFields() : textfields.selectedFields(),
            selectedImages = gallery.selectedImages();

        _credits = bulk ? bulkCredits : settings.credits();

        if (_credits.generation === undefined) {
            return total;
        }

        if (bulk) {
            _selector = '.bulk_credit_computation';
        } else {
            _selector = '.credit_computation';
        }

        _rules = rules;

        if (rewrite) {
            $(_selector + '_rewrite span').html('');
        } else {
            $(_selector + ' span').html('');
        }

        for (const field of selectedFields) {
            switch (field) {
                case 'page_title':
                    if (rewrite) {
                        $(_selector + '_rewrite .' + field).html("credits: " + getPageTitleCredits(field, rewrite));
                    } else {
                        $(_selector + ' .' + field).html("credits: " + getPageTitleCredits(field, rewrite));
                    }
                    total += getPageTitleCredits(field, rewrite);
                    break;
                case 'page_description':
                    if (rewrite) {
                        $(_selector + '_rewrite .' + field).html("credits: " + getPageDescriptionCredits(field, rewrite));
                    } else {
                        $(_selector + ' .' + field).html("credits: " + getPageDescriptionCredits(field, rewrite));
                    }
                    total += getPageDescriptionCredits(field, rewrite);
                    break;
                case 'product_description':
                    total += getProductDescriptionCredits(field, rewrite, bulk);
                    break;
                case 'short_product_description':
                    total += getProductExcerptCredits(field, rewrite, bulk);
                    break;
                case 'open_graph':
                    if (rewrite) {
                        $(_selector + '_rewrite .' + field).html("credits: " + getOpenGraphCredits(field, rewrite));
                    } else {
                        $(_selector + ' .' + field).html("credits: " + getOpenGraphCredits(field, rewrite));
                    }
                    total += getOpenGraphCredits(field, rewrite);
                    break;
            }
        }

        if (premium.isPremium()) {
            if (!bulk &&
                selectedImages.length > 0 && (
                settings.submitAction() !== 'rewrite' &&
                !referenceProducts.referenceProductChecked()
            )) {
                total += Math.ceil((selectedImages.length / _rules.maxImageAltTextPerRequest) * _credits.altText);
            }
        }

        if (bulk) {
            total *= prompt.selectionsCount();

            if (selectedFields.includes('alt_text')) {
                prompt.selectionImagesCount().forEach(function (value) {
                    total += Math.ceil((value / _rules.maxImageAltTextPerRequest) * _credits.altText);
                });
            }
        }

        return total;
    }

    return {
        getCredits: getCredits
    };
});
